@@ -211,12 +211,14 @@ void carray_set_from_rarray(double *a, VALUE ary)
   VALUE val;
   if (CLASS_OF(ary) == rb_cRange) ary = rb_gsl_range2ary(ary);
   Check_Type(ary, T_ARRAY);
-  size = RARRAY(ary)->len;
+  //  size = RARRAY(ary)->len;
+  size = RARRAY_LEN(ary);
   if (size == 0) return;
   for (i = 0; i < size; i++) {
     val = rb_ary_entry(ary, i);
     Need_Float(val);
-    a[i] = RFLOAT(val)->value;
+    a[i] = NUM2DBL(val);
+    //    a[i] = RFLOAT(val)->value;
   }
 }
 
@@ -405,7 +407,7 @@ void gsl_matrix_complex_mul_vector(gsl_vector_complex *vnew,
 }
 
 /*****/
-
+#ifndef GSL_1_12_LATER
 int gsl_vector_complex_add(gsl_vector_complex *cv, const gsl_vector_complex *cv2)
 {
   size_t i;
@@ -482,26 +484,27 @@ int gsl_vector_complex_div(gsl_vector_complex *cv, const gsl_vector_complex *cv2
   }
   return 0;
 }
+#endif
 
-void get_range_beg_en_n(VALUE range, int *beg, int *en, size_t *n, int *step);
 VALUE rb_gsl_range2ary(VALUE obj)
 {
-  int beg, en, i, step;
-  size_t n;
+  //  double beg, en;
+  //  size_t n;
+  //  int step;
   VALUE ary;
   if (CLASS_OF(obj) != rb_cRange) 
     rb_raise(rb_eTypeError, "wrong argument type %s (Range expected)",
 	     rb_class2name(CLASS_OF(obj)));
-  get_range_beg_en_n(obj, &beg, &en, &n, &step);
-  ary = rb_ary_new2(n);
-  for (i = 0; i < n; i++) rb_ary_store(ary, i, INT2FIX(beg+i));
+  ary = rb_funcall(obj, rb_gsl_id_to_a, 0);
   return ary;
 }
 
+void get_range_beg_en_n(VALUE range, double *beg, double *en, size_t *n, int *step);
 VALUE rb_gsl_range2vector(VALUE obj)
 {
-  int beg, en, i, step;
+  double beg, en;
   size_t n;
+  int i, step;
   gsl_vector *v;
   if (CLASS_OF(obj) != rb_cRange) 
     rb_raise(rb_eTypeError, "wrong argument type %s (Range expected)",
@@ -512,6 +515,7 @@ VALUE rb_gsl_range2vector(VALUE obj)
   return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
 }
 
+void get_range_int_beg_en_n(VALUE range, int *beg, int *en, size_t *n, int *step);
 VALUE rb_gsl_range2vector_int(VALUE obj)
 {
   int beg, en, i, step;
@@ -520,7 +524,7 @@ VALUE rb_gsl_range2vector_int(VALUE obj)
   if (CLASS_OF(obj) != rb_cRange) 
     rb_raise(rb_eTypeError, "wrong argument type %s (Range expected)",
 	     rb_class2name(CLASS_OF(obj)));
-  get_range_beg_en_n(obj, &beg, &en, &n, &step);
+  get_range_int_beg_en_n(obj, &beg, &en, &n, &step);
   v = gsl_vector_int_alloc(n);
   for (i = 0; i < n; i++) gsl_vector_int_set(v, i, beg+i);
   return Data_Wrap_Struct(cgsl_vector_int, 0, gsl_vector_int_free, v);
@@ -650,6 +654,7 @@ void Init_gsl_array(VALUE module)
   Init_gsl_combination(module);
 #endif
   Init_gsl_array_complex(module);
+  Init_gsl_matrix_nmf();
 
   rb_define_method(cgsl_vector_view_ro, "set", rb_gsl_obj_read_only, -1);
   rb_define_method(cgsl_matrix_view_ro, "set", rb_gsl_obj_read_only, -1);
