@@ -98,19 +98,33 @@ static VALUE rb_gsl_bspline_eval(int argc, VALUE *argv, VALUE obj)
 
 	return vB;
 }
-void Init_bspline(VALUE module)
+#ifdef GSL_1_13_LATER
+static VALUE rb_gsl_bspline_greville_abscissa(VALUE obj, VALUE i)
 {
-	cBSWS = rb_define_class_under(module, "BSpline", cGSL_Object);
-	
-	rb_define_singleton_method(cBSWS, "alloc", rb_gsl_bspline_alloc, 2);
-	
-	rb_define_method(cBSWS, "ncoeffs", rb_gsl_bspline_ncoeffs, 0);
-	rb_define_method(cBSWS, "order", rb_gsl_bspline_order, 0);
-	rb_define_method(cBSWS, "nbreak", rb_gsl_bspline_nbreak, 0);
-	rb_define_method(cBSWS, "breakpoint", rb_gsl_bspline_breakpoint, 1);
-	rb_define_method(cBSWS, "knots", rb_gsl_bspline_knots, 1);
-	rb_define_method(cBSWS, "knots_uniform", rb_gsl_bspline_knots_uniform, -1);
-	rb_define_singleton_method(cBSWS, "knots_uniform", rb_gsl_bspline_knots_uniform, -1);	
-	rb_define_method(cBSWS, "eval", rb_gsl_bspline_eval, -1);	
+  gsl_bspline_workspace *w;
+  Data_Get_Struct(obj, gsl_bspline_workspace, w);
+  return rb_float_new(gsl_bspline_greville_abscissa(i, w));
+}
+#endif
+
+void Init_bspline(VALUE module)
+{
+  cBSWS = rb_define_class_under(module, "BSpline", cGSL_Object);
+  
+  rb_define_singleton_method(cBSWS, "alloc", rb_gsl_bspline_alloc, 2);
+  
+  rb_define_method(cBSWS, "ncoeffs", rb_gsl_bspline_ncoeffs, 0);
+  rb_define_method(cBSWS, "order", rb_gsl_bspline_order, 0);
+  rb_define_method(cBSWS, "nbreak", rb_gsl_bspline_nbreak, 0);
+  rb_define_method(cBSWS, "breakpoint", rb_gsl_bspline_breakpoint, 1);
+  rb_define_method(cBSWS, "knots", rb_gsl_bspline_knots, 1);
+  rb_define_method(cBSWS, "knots_uniform", rb_gsl_bspline_knots_uniform, -1);
+  rb_define_singleton_method(cBSWS, "knots_uniform", rb_gsl_bspline_knots_uniform, -1);	
+  rb_define_method(cBSWS, "eval", rb_gsl_bspline_eval, -1);	
+
+#ifdef GSL_1_13_LATER
+  rb_define_method(cBSWS, "greville_abscissa", rb_gsl_bspline_greville_abscissa, 1);
+#endif
+
 }
 #endif
