@@ -2,32 +2,32 @@ require 'mkmf'
 
 
 module GSL
-	class Version
-		def initialize(str)
-			@str = str
-			@ary = str.split(".").collect { |elm| elm.to_i }
-		end
-		def to_s; @str; end
-		def inspect; @str; end
-		def >=(ver)
-			ary2 = ver.split(".").collect { |elm| elm.to_i }
-			if @ary[0] > ary2[0]; return true; end			
-			if @ary[0] < ary2[0]; return false; end
-			if @ary[1] > ary2[1]; return true; end
-			if @ary[1] < ary2[1]; return false; end
-			if @ary.size < ary2.size; return false; end
-			if @ary.size == 3 and ary2.size == 3
-				if @ary[2] < ary2[2]; return false; end
-			end		
-			return true
-		end
-		def <(ver)
-			ary2 = ver.split(".").collect { |elm| elm.to_i }
-			if @ary[0] >= ary2[0]; return false; end
-			if @ary[0] >= ary2[0]; return false; end
-			return true
-		end
-	end
+  class Version
+    def initialize(str)
+      @str = str
+      @ary = str.split(".").collect { |elm| elm.to_i }
+    end
+    def to_s; @str; end
+    def inspect; @str; end
+    def >=(ver)
+      ary2 = ver.split(".").collect { |elm| elm.to_i }
+      if @ary[0] > ary2[0]; return true; end			
+      if @ary[0] < ary2[0]; return false; end
+      if @ary[1] > ary2[1]; return true; end
+      if @ary[1] < ary2[1]; return false; end
+      if @ary.size < ary2.size; return false; end
+      if @ary.size == 3 and ary2.size == 3
+        if @ary[2] < ary2[2]; return false; end
+      end		
+      return true
+    end
+    def <(ver)
+      ary2 = ver.split(".").collect { |elm| elm.to_i }
+      if @ary[0] >= ary2[0]; return false; end
+      if @ary[0] >= ary2[0]; return false; end
+      return true
+    end
+  end
 end
 
 if /mingw/ =~ RUBY_PLATFORM
@@ -43,7 +43,7 @@ def gsl_config()
     puts(cflags)
     $CFLAGS += " " + cflags
   end
-   
+  
   IO.popen("#{GSL_CONFIG} --libs") do |f|
     libs = f.gets.chomp
     dir_config("cblas")
@@ -116,12 +116,24 @@ def check_version(configfile)
       configfile.printf("#ifndef GSL_CONST_OLD\n#define GSL_CONST_OLD\n#endif\n")
     end
 
+    if ver >= "1.11"
+      configfile.printf("#ifndef GSL_1_11_LATER\n#define GSL_1_11_LATER\n#endif\n")
+    end    
+
+   if ver >= "1.12.90"
+      configfile.printf("#ifndef GSL_1_13_LATER\n#define GSL_1_13_LATER\n#endif\n")
+    end
+
+    if ver >= "1.14"
+      configfile.printf("#ifndef GSL_1_14_LATER\n#define GSL_1_14_LATER\n#endif\n")
+    end    
+    
   end
 end
 
 #####
 
-$CFLAGS = " -Wall -I../include"
+$CFLAGS = " -Wall -I../include "
 
 begin
   RB_GSL_CONFIG = File.open("../include/rb_gsl_config.h", "w")
@@ -148,10 +160,10 @@ begin
     have_library("ool")
   end
   
-  if have_header("gsl_tensor/gsl_tensor.h")
-    have_library("gsltensor")
+  if have_header("tensor/tensor.h")
+    have_library("tensor")
   end
- 
+
   if have_header("jacobi.h")
     have_library("jacobi")
   end
@@ -169,10 +181,15 @@ begin
     RB_GSL_CONFIG.printf("#ifndef HAVE_EIGEN_FRANCIS\n#define HAVE_EIGEN_FRANCIS\n#endif\n")
   end
 
-	if have_header("ndlinear/gsl_multifit_ndlinear.h")
-		have_library("ndlinear")
-	end
-	
+  if have_header("ndlinear/gsl_multifit_ndlinear.h")
+    have_library("ndlinear")
+  end
+
+# Added 2009/Apr/20
+  if have_header("alf/alf.h")
+    have_library("alf")
+  end
+  
   begin
     print("checking rb-gsl version...")
     IO.popen("cat ../VERSION") do |f|
@@ -218,6 +235,16 @@ end
 
 #narray_config = dir_config("narray")
 narray_config = dir_config('narray',$sitearchdir,$sitearchdir)
+# Try to find narray with RubyGems
+begin
+  require 'rubygems'
+  na_gemspec=Gem.searcher.find('narray.h')
+  if na_gemspec
+    narray_config = File.join(na_gemspec.full_gem_path, na_gemspec.require_path)
+    $CPPFLAGS = " -I#{narray_config} "+$CPPFLAGS
+  end
+rescue LoadError
+end
 have_narray_h = have_header("narray.h")
 if narray_config
   if RUBY_PLATFORM =~ /cygwin|mingw/
@@ -237,19 +264,19 @@ end
 
 File.open("../lib/gsl.rb", "w") do |file|
   if have_narray_h
-    file.print("require\(\"narray\"\)\n")
+    file.print("require('narray')\n")
   end
-#  file.print("require\(\"rb_gsl\"\)\ninclude GSL\n")
-  file.print("require\(\"rb_gsl\"\)\n")  
-  file.print("require\(\"gsl/oper.rb\"\)\n")
+#  file.print("require('rb_gsl')\ninclude GSL\n")
+  file.print("require('rb_gsl')\n")  
+  file.print("require('gsl/oper.rb')\n")
 end
 
 File.open("../lib/rbgsl.rb", "w") do |file|
   if have_narray_h
-    file.print("require\(\"narray\"\)\n")
+    file.print("require('narray')\n")
   end
-  file.print("require\(\"rb_gsl\"\)\n")
-  file.print("require\(\"gsl/oper.rb\"\)\n")
+  file.print("require('rb_gsl')\n")
+  file.print("require('gsl/oper.rb')\n")
 end
 
 srcs = Dir.glob("*.c") - ["vector_source.c", "matrix_source.c", "tensor_source.c", "poly_source.c", "block_source.c"]
