@@ -12,7 +12,7 @@
 #include "rb_gsl.h"
 #include <gsl/gsl_machine.h>
 
-ID rb_gsl_id_beg, rb_gsl_id_end, rb_gsl_id_excl;
+ID rb_gsl_id_beg, rb_gsl_id_end, rb_gsl_id_excl, rb_gsl_id_to_a;
 static ID rb_gsl_id_name, rb_gsl_id_size;
 VALUE cGSL_Object;
 static void rb_gsl_define_intern(VALUE module);
@@ -46,7 +46,11 @@ static VALUE rb_gsl_object_info(VALUE obj)
   char buf[256];
   VALUE s;
   sprintf(buf, "Class:      %s\n", rb_class2name(CLASS_OF(obj)));
+#ifdef RUBY_1_9_LATER
+  sprintf(buf, "%sSuperClass: %s\n", buf, rb_class2name(RCLASS_SUPER(CLASS_OF(obj))));
+#else
   sprintf(buf, "%sSuperClass: %s\n", buf, rb_class2name(RCLASS(CLASS_OF(obj))->super));
+#endif
   s = rb_rescue(rb_gsl_call_name, obj, rb_gsl_call_rescue, obj);
   if (s) sprintf(buf, "%sType:       %s\n", buf, STR2CSTR(s));
   s = rb_rescue(rb_gsl_call_size, obj, rb_gsl_call_rescue, obj);
@@ -54,6 +58,12 @@ static VALUE rb_gsl_object_info(VALUE obj)
   return rb_str_new2(buf);
 }
 
+static VALUE rb_gsl_not_implemeted(VALUE obj)
+{
+  rb_raise(rb_eNotImpError, "%s#dup is not implemented", rb_class2name(CLASS_OF(obj)));
+  return Qnil;
+}
+
 void Init_rb_gsl()
 {
   VALUE mgsl;
@@ -62,6 +72,10 @@ void Init_rb_gsl()
   cGSL_Object = rb_define_class_under(mgsl, "Object", rb_cObject);
   rb_define_method(cGSL_Object, "inspect", rb_gsl_object_inspect, 0);
   rb_define_method(cGSL_Object, "info", rb_gsl_object_info, 0);
+  // Override Object#dup to prevent invalid dup-ing of GSL_Objects.
+  // Subclasses (e.g. GSL::Vector) must provide their own implementation of
+  // #dup if desired.
+  rb_define_method(cGSL_Object, "dup", rb_gsl_not_implemeted, 0);
 
   rb_gsl_define_intern(mgsl);
 
@@ -88,7 +102,7 @@ void Init_rb_gsl()
   Init_gsl_eigen(mgsl);
 
   Init_gsl_fft(mgsl);
-  /*  Init_gsl_signal(mgsl);*/
+    Init_gsl_signal(mgsl);
   Init_gsl_function(mgsl);
   Init_gsl_integration(mgsl);
 
@@ -138,9 +152,9 @@ void Init_rb_gsl()
 
   rb_gsl_define_const(mgsl);
 
-#ifdef HAVE_GSL_TENSOR_GSL_TENSOR_H
-  Init_gsl_tensor_init(mgsl);
-  Init_gsl_tensor_int_init(mgsl);
+#ifdef HAVE_TENSOR_TENSOR_H
+  Init_tensor_init(mgsl);
+  Init_tensor_int_init(mgsl);
 #endif
 
   Init_gsl_graph(mgsl);
@@ -168,6 +182,16 @@ void Init_rb_gsl()
 	Init_bspline(mgsl);
 #endif
 
+#ifdef HAVE_ALF_ALF_H
+	Init_alf(mgsl);
+#endif
+
+	Init_geometry(mgsl);
+
+#ifdef GSL_1_14_LATER
+	Init_multiset(mgsl);
+#endif
+
   rb_gsl_define_methods(mgsl);
 }
 
@@ -178,6 +202,7 @@ static void rb_gsl_define_intern(VALUE module)
   rb_gsl_id_beg  = rb_intern("begin");
   rb_gsl_id_end  = rb_intern("end");
   rb_gsl_id_excl = rb_intern("excl");
+  rb_gsl_id_to_a = rb_intern("to_a");
   rb_gsl_id_name  = rb_intern("name");
   rb_gsl_id_size  = rb_intern("size");
 }
@@ -223,11 +248,16 @@ static void rb_gsl_define_const(VALUE module)
   rb_define_const(module, "ROOT6_DBL_MIN", rb_float_new(GSL_ROOT6_DBL_MIN));
   rb_define_const(module, "LOG_DBL_MIN", rb_float_new(GSL_LOG_DBL_MIN));
 
+#ifdef GSL_1_14_LATER
+  rb_define_const(module, "MAJOR_VERSION", INT2FIX(GSL_MAJOR_VERSION));
+  rb_define_const(module, "MINOR_VERSION", INT2FIX(GSL_MINOR_VERSION));
+#endif
+
 }
 
 static VALUE rb_gsl_have_tensor(VALUE module)
 {
-#ifdef HAVE_GSL_TENSOR_GSL_TENSOR_H
+#ifdef HAVE_TENSOR_TENSOR_H
   return Qtrue;
 #else
   return Qfalse;
