@@ -301,7 +301,7 @@ static VALUE rb_gsl_histogram_set_ranges_uniform(int argc, VALUE *argv, VALUE ob
     xmax = NUM2DBL(argv[1]);
     break;
   default:
-    rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)");
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
     break;
   }
   Data_Get_Struct(obj, gsl_histogram, h);
@@ -354,7 +354,8 @@ static VALUE rb_gsl_histogram_accumulate(int argc, VALUE *argv, VALUE obj)
   }
   Data_Get_Struct(obj, gsl_histogram, h);
   if (TYPE(argv[0]) == T_ARRAY) {
-    for (i = 0; i < RARRAY(argv[0])->len; i++)
+    //    for (i = 0; i < RARRAY(argv[0])->len; i++)
+    for (i = 0; i < RARRAY_LEN(argv[0]); i++)
       gsl_histogram_accumulate(h, NUM2DBL(rb_ary_entry(argv[0], i)), weight);
   } else if (VECTOR_P(argv[0])) {
     Data_Get_Struct(argv[0], gsl_vector, v);
@@ -1143,7 +1144,9 @@ static int Gaussian_f(const gsl_vector *v, void *params, gsl_vector *f)
       rb_raise(rb_eIndexError, "wrong index");
     xi = (xl + xh)/2.0;
     yi = h->bin[i];
-    sqw = sqrt(yi);
+    //    sqw = sqrt(yi);
+    if (yi >= 1.0) sqw = 1.0/sqrt(yi);
+    else sqw = 1.0;
     gsl_vector_set(f, i-binstart, (amp*exp(-(xi - mu)*(xi - mu)/var/2.0) - yi)*sqw);
   }
   return GSL_SUCCESS;
@@ -1167,7 +1170,9 @@ static int Gaussian_df(const gsl_vector *v, void *params, gsl_matrix *J)
       rb_raise(rb_eIndexError, "wrong index");
     xi = (xl + xh)/2.0;
     yi = h->bin[i];
-    sqw = sqrt(yi);
+    //    sqw = sqrt(yi);
+    if (yi >= 1.0) sqw = 1.0/sqrt(yi);
+    else sqw = 1.0;
     y = exp(-(xi - mu)*(xi - mu)/var/2.0);
     gsl_matrix_set(J, i-binstart, 0, amp*y*(xi - mu)*(xi - mu)/2/var/var*sqw);
     gsl_matrix_set(J, i-binstart, 1, amp*y*(xi - mu)/var*sqw);
@@ -1215,9 +1220,9 @@ static VALUE rb_gsl_histogram_fit_gaussian(int argc, VALUE *argv, VALUE obj)
     break;
   }
   x = gsl_vector_alloc(p);
-  gsl_vector_set(x, 0, 1);   /* initial values, var = 1 */
-  gsl_vector_set(x, 1, 0.0);   /* mu = 0 */
-  gsl_vector_set(x, 2, 1);   /* amp = 1 */
+  gsl_vector_set(x, 0, gsl_pow_2(gsl_histogram_sigma(h)));   /* initial values, var = 1 */
+  gsl_vector_set(x, 1, gsl_histogram_mean(h));   /* mu = 0 */
+  gsl_vector_set(x, 2, gsl_histogram_max_val(h));   /* amp = 1 */
   hh.h = h;
   hh.binstart = binstart;
   hh.binend = binend;
@@ -1261,6 +1266,275 @@ static VALUE rb_gsl_histogram_fit_gaussian(int argc, VALUE *argv, VALUE obj)
 		     rb_float_new(chi2), INT2FIX(dof));
 }
 
+static int Rayleigh_f(const gsl_vector *v, void *params, gsl_vector *f);
+static int Rayleigh_df(const gsl_vector *v, void *params, gsl_matrix * J);
+static int Rayleigh_f(const gsl_vector *v, void *params, gsl_vector *f)
+{
+  struct fit_histogram *hh;
+  gsl_histogram *h = NULL;
+  double amp, var, xl, xh, xi, yi, sqw;
+  size_t i, binstart, binend;
+  hh = (struct fit_histogram *) params;
+  h = hh->h;
+  binstart = hh->binstart;
+  binend = hh->binend;
+  var = gsl_vector_get(v, 0);
+  amp = gsl_vector_get(v, 1);
+  for (i = binstart; i <= binend; i++) {
+    if (gsl_histogram_get_range(h, i, &xl, &xh))
+      rb_raise(rb_eIndexError, "wrong index");
+    xi = (xl + xh)/2.0;
+    yi = h->bin[i];
+    sqw = sqrt(yi);
+    gsl_vector_set(f, i-binstart, (amp*xi*exp(-xi*xi/var/2.0) - yi)*sqw);
+  }
+  return GSL_SUCCESS;
+}
+
+static int Rayleigh_df(const gsl_vector *v, void *params, gsl_matrix *J)
+{
+  struct fit_histogram *hh;
+  gsl_histogram *h = NULL;
+  double amp, var, xl, xh, xi, yi, y, sqw;
+  size_t i, binstart, binend;
+  hh = (struct fit_histogram *) params;
+  h = hh->h;
+  binstart = hh->binstart;
+  binend = hh->binend;
+  var = gsl_vector_get(v, 0);
+  amp = gsl_vector_get(v, 1);
+  for (i = binstart; i <= binend; i++) {
+    if (gsl_histogram_get_range(h, i, &xl, &xh))
+      rb_raise(rb_eIndexError, "wrong index");
+    xi = (xl + xh)/2.0;
+    yi = h->bin[i];
+    sqw = sqrt(yi);
+    y = xi*exp(-xi*xi/var/2.0);
+    gsl_matrix_set(J, i-binstart, 0, amp*y*xi*xi/2/var/var*sqw);
+    gsl_matrix_set(J, i-binstart, 1, y*sqw);
+  }
+  return GSL_SUCCESS;
+}
+
+static int Rayleigh_fdf(const gsl_vector *v, void *params, gsl_vector *f, 
+			gsl_matrix *J)
+{
+  Rayleigh_f(v, params, f);
+  Rayleigh_df(v, params, J);
+  return GSL_SUCCESS;
+}
+
+
+static VALUE rb_gsl_histogram_fit_rayleigh(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_histogram *h = NULL;
+  struct fit_histogram hh;
+  const gsl_multifit_fdfsolver_type *T;
+  gsl_multifit_fdfsolver *s;
+  int status;
+  size_t iter = 0, binstart, binend;
+  size_t n, dof;      /* # of data points */
+  size_t p = 2;  /* # of fitting parameters */
+  gsl_multifit_function_fdf f;
+  gsl_matrix *covar = NULL;
+  gsl_vector *x = NULL;
+  double sigma, height, errs, errh, chi2;
+  Data_Get_Struct(obj, gsl_histogram, h);
+  binstart = 0;
+  binend = h->n - 1;
+  switch (argc) {
+  case 2:
+    CHECK_FIXNUM(argv[0]); CHECK_FIXNUM(argv[1]);
+    binstart = FIX2INT(argv[0]);
+    binend = FIX2INT(argv[1]);
+    if (binend >= h->n) binend = h->n - 1;
+    break;
+  case 0:
+    break;
+  default:
+    rb_raise(rb_eArgError, "too many arguments (%d for 0 or 2)", argc);
+    break;
+  }
+  x = gsl_vector_alloc(p);
+  gsl_vector_set(x, 0, gsl_pow_2(gsl_histogram_sigma(h)));   /* initial values, var = 1 */
+  gsl_vector_set(x, 1, gsl_histogram_max_val(h));   
+  hh.h = h;
+  hh.binstart = binstart;
+  hh.binend = binend;
+  n = binend - binstart + 1;
+
+  covar = gsl_matrix_alloc(p, p);
+
+  f.f = Rayleigh_f;
+  f.df = Rayleigh_df;
+  f.fdf = Rayleigh_fdf;
+  f.n = n;
+  f.p = p;
+  f.params = &hh;
+
+  T = gsl_multifit_fdfsolver_lmsder;
+  s = gsl_multifit_fdfsolver_alloc(T, n, p);
+  gsl_multifit_fdfsolver_set(s, &f, x);
+
+  do {
+    iter++;
+    status = gsl_multifit_fdfsolver_iterate(s);
+    if (status) break;
+    status = gsl_multifit_test_delta(s->dx, s->x, 1e-4, 1e-4);
+  } while (status == GSL_CONTINUE);
+  sigma = sqrt(gsl_vector_get(s->x, 0));
+  height = gsl_vector_get(s->x, 1)*sigma*sigma;
+  gsl_multifit_covar(s->J, 0.0, covar);
+  chi2 = gsl_pow_2(gsl_blas_dnrm2(s->f));   /* not reduced chi-square */
+  dof = n - p;
+  errs = sqrt(chi2/dof*gsl_matrix_get(covar, 0, 0))/sigma/2;
+  errh = sqrt(chi2/dof*gsl_matrix_get(covar, 1, 1));
+
+  gsl_multifit_fdfsolver_free(s);
+  gsl_vector_free(x);
+  gsl_matrix_free(covar);
+  return rb_ary_new3(6, rb_float_new(sigma), 
+		     rb_float_new(height), rb_float_new(errs),
+		     rb_float_new(errh),
+		     rb_float_new(chi2), INT2FIX(dof));
+}
+
+/*
+ * y(x) = Amp*exp(-a*x)
+ */
+static int xExponential_f(const gsl_vector *v, void *params, gsl_vector *f);
+static int xExponential_df(const gsl_vector *v, void *params, gsl_matrix * J);
+static int xExponential_f(const gsl_vector *v, void *params, gsl_vector *f)
+{
+  struct fit_histogram *hh;
+  gsl_histogram *h = NULL;
+  double amp, b, xl, xh, xi, yi, sqw;
+  size_t i, binstart, binend;
+  hh = (struct fit_histogram *) params;
+  h = hh->h;
+  binstart = hh->binstart;
+  binend = hh->binend;
+  b = gsl_vector_get(v, 0);
+  amp = gsl_vector_get(v, 1);
+  for (i = binstart; i <= binend; i++) {
+    if (gsl_histogram_get_range(h, i, &xl, &xh))
+      rb_raise(rb_eIndexError, "wrong index");
+    xi = (xl + xh)/2.0;
+    yi = h->bin[i];
+    sqw = sqrt(yi);
+    gsl_vector_set(f, i-binstart, (amp*xi*exp(-b*xi) - yi)*sqw);
+  }
+  return GSL_SUCCESS;
+}
+
+static int xExponential_df(const gsl_vector *v, void *params, gsl_matrix *J)
+{
+  struct fit_histogram *hh;
+  gsl_histogram *h = NULL;
+  double amp, b, xl, xh, xi, yi, y, sqw;
+  size_t i, binstart, binend;
+  hh = (struct fit_histogram *) params;
+  h = hh->h;
+  binstart = hh->binstart;
+  binend = hh->binend;
+  b = gsl_vector_get(v, 0);
+  amp = gsl_vector_get(v, 1);
+  for (i = binstart; i <= binend; i++) {
+    if (gsl_histogram_get_range(h, i, &xl, &xh))
+      rb_raise(rb_eIndexError, "wrong index");
+    xi = (xl + xh)/2.0;
+    yi = h->bin[i];
+    sqw = sqrt(yi);
+    y = xi*exp(-b*xi);
+    gsl_matrix_set(J, i-binstart, 0, -amp*y*xi*sqw);
+    gsl_matrix_set(J, i-binstart, 1, y*sqw);
+  }
+  return GSL_SUCCESS;
+}
+
+static int xExponential_fdf(const gsl_vector *v, void *params, gsl_vector *f, 
+			gsl_matrix *J)
+{
+  xExponential_f(v, params, f);
+  xExponential_df(v, params, J);
+  return GSL_SUCCESS;
+}
+
+
+static VALUE rb_gsl_histogram_fit_xexponential(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_histogram *h = NULL;
+  struct fit_histogram hh;
+  const gsl_multifit_fdfsolver_type *T;
+  gsl_multifit_fdfsolver *s;
+  int status;
+  size_t iter = 0, binstart, binend;
+  size_t n, dof;      /* # of data points */
+  size_t p = 2;  /* # of fitting parameters */
+  gsl_multifit_function_fdf f;
+  gsl_matrix *covar = NULL;
+  gsl_vector *x = NULL;
+  double b, height, errs, errh, chi2;
+  Data_Get_Struct(obj, gsl_histogram, h);
+  binstart = 0;
+  binend = h->n - 1;
+  switch (argc) {
+  case 2:
+    CHECK_FIXNUM(argv[0]); CHECK_FIXNUM(argv[1]);
+    binstart = FIX2INT(argv[0]);
+    binend = FIX2INT(argv[1]);
+    if (binend >= h->n) binend = h->n - 1;
+    break;
+  case 0:
+    break;
+  default:
+    rb_raise(rb_eArgError, "too many arguments (%d for 0 or 2)", argc);
+    break;
+  }
+  x = gsl_vector_alloc(p);
+  gsl_vector_set(x, 0, gsl_histogram_sigma(h));   /* initial values, var = 1 */
+  gsl_vector_set(x, 1, gsl_histogram_max_val(h));   
+  hh.h = h;
+  hh.binstart = binstart;
+  hh.binend = binend;
+  n = binend - binstart + 1;
+
+  covar = gsl_matrix_alloc(p, p);
+
+  f.f = xExponential_f;
+  f.df = xExponential_df;
+  f.fdf = xExponential_fdf;
+  f.n = n;
+  f.p = p;
+  f.params = &hh;
+
+  T = gsl_multifit_fdfsolver_lmsder;
+  s = gsl_multifit_fdfsolver_alloc(T, n, p);
+  gsl_multifit_fdfsolver_set(s, &f, x);
+
+  do {
+    iter++;
+    status = gsl_multifit_fdfsolver_iterate(s);
+    if (status) break;
+    status = gsl_multifit_test_delta(s->dx, s->x, 1e-4, 1e-4);
+  } while (status == GSL_CONTINUE);
+  b = gsl_vector_get(s->x, 0);
+  height = gsl_vector_get(s->x, 1);
+  gsl_multifit_covar(s->J, 0.0, covar);
+  chi2 = gsl_pow_2(gsl_blas_dnrm2(s->f));   /* not reduced chi-square */
+  dof = n - p;
+  errs = sqrt(chi2/dof*gsl_matrix_get(covar, 0, 0));
+  errh = sqrt(chi2/dof*gsl_matrix_get(covar, 1, 1));
+
+  gsl_multifit_fdfsolver_free(s);
+  gsl_vector_free(x);
+  gsl_matrix_free(covar);
+  return rb_ary_new3(6, rb_float_new(b), 
+		     rb_float_new(height), rb_float_new(errs),
+		     rb_float_new(errh),
+		     rb_float_new(chi2), INT2FIX(dof));
+}
+
 static VALUE rb_gsl_histogram_fit(int argc, VALUE *argv, VALUE obj)
 {
   char fittype[32];
@@ -1273,6 +1547,10 @@ static VALUE rb_gsl_histogram_fit(int argc, VALUE *argv, VALUE obj)
     return rb_gsl_histogram_fit_power(argc-1, argv+1, obj);
   } else if (str_head_grep(fittype, "gaus") == 0) {
     return rb_gsl_histogram_fit_gaussian(argc-1, argv+1, obj);
+ } else if (str_head_grep(fittype, "rayleigh") == 0) {
+    return rb_gsl_histogram_fit_rayleigh(argc-1, argv+1, obj);
+  } else if (str_head_grep(fittype, "xexp") == 0) {
+    return rb_gsl_histogram_fit_xexponential(argc-1, argv+1, obj);
   } else {
     rb_raise(rb_eRuntimeError, 
 	     "unknown fitting type %s (exp, power, gaus expected)", fittype);
@@ -1489,6 +1767,75 @@ static VALUE rb_gsl_histogram_reverse(VALUE obj)
   return Data_Wrap_Struct(cgsl_histogram, 0, gsl_histogram_free, hnew);
 }
 
+/* The functions below are not included in GSL */
+/*
+ * Returns an x value at which the histogram integration
+ * reaches the given percentile. The x value is calculated
+ * by an interpolation between the ranges in which the percentile
+ * is found.
+ */
+static double histogram_percentile(const gsl_histogram *h, double f)
+{
+  double sum = gsl_histogram_sum(h), sf;
+  double val, s = 0, x;
+  double ri, ri1;
+  size_t i;
+  sf = sum * f;
+  for (i = 0; i < h->n; i++) {
+    val = gsl_histogram_get(h, i);
+    if ((s+val) > sf) break;
+    s += val;
+  }
+  ri = h->range[i];
+  ri1 = h->range[i+1];
+  x = (sf - s)*(ri1 - ri)/val + ri;
+  return x;
+}
+
+static double histogram_median(const gsl_histogram *h)
+{
+  return histogram_percentile(h, 0.5);
+}
+
+static VALUE rb_gsl_histogram_percentile(VALUE obj, VALUE f)
+{
+  gsl_histogram *h;
+  Data_Get_Struct(obj, gsl_histogram, h);
+  return rb_float_new(histogram_percentile(h, NUM2DBL(f)));
+}
+
+static VALUE rb_gsl_histogram_median(VALUE obj)
+{
+  gsl_histogram *h;
+  Data_Get_Struct(obj, gsl_histogram, h);
+  return rb_float_new(histogram_median(h));
+}
+
+static double histogram_percentile_inv(const gsl_histogram *h, double x)
+{
+  double sum = gsl_histogram_sum(h);
+  double val, s = 0;
+  double ri, ri1, q;
+  size_t i;
+
+  for (i = 0; i < h->n; i++) {
+    val = gsl_histogram_get(h, i);
+    if (h->range[i+1] > x) break;
+    s += val;
+  }
+  ri = h->range[i];
+  ri1 = h->range[i+1];
+  q = s + val/(ri1 - ri)*(x - ri);
+  return q/sum;
+}
+
+static VALUE rb_gsl_histogram_percentile_inv(VALUE obj, VALUE x)
+{
+  gsl_histogram *h;
+  Data_Get_Struct(obj, gsl_histogram, h);
+  return rb_float_new(histogram_percentile_inv(h, NUM2DBL(x)));
+}
+
 void Init_gsl_histogram(VALUE module)
 {
   VALUE cgsl_histogram_pdf;
@@ -1507,9 +1854,9 @@ void Init_gsl_histogram(VALUE module)
   rb_define_singleton_method(cgsl_histogram, "[]", rb_gsl_histogram_alloc, -1);
 
   rb_define_singleton_method(cgsl_histogram, "alloc_uniform", 
-			     rb_gsl_histogram_alloc_uniform, 3);
+			     rb_gsl_histogram_alloc_uniform, -1);
   rb_define_singleton_method(cgsl_histogram, "new_uniform", 
-			     rb_gsl_histogram_alloc_uniform, 3);
+			     rb_gsl_histogram_alloc_uniform, -1);
 
   rb_define_singleton_method(cgsl_histogram, "alloc_with_min_max_step", 
 			     rb_gsl_histogram_alloc_with_min_max_step, 3);
@@ -1623,8 +1970,12 @@ void Init_gsl_histogram(VALUE module)
 		   rb_gsl_histogram_fit_gaussian, -1);
   rb_define_method(cgsl_histogram, "fit_exponential", 
 		   rb_gsl_histogram_fit_exponential, -1);
+  rb_define_method(cgsl_histogram, "fit_xexponential", 
+		   rb_gsl_histogram_fit_xexponential, -1);
   rb_define_method(cgsl_histogram, "fit_power", 
 		   rb_gsl_histogram_fit_power, -1);
+  rb_define_method(cgsl_histogram, "fit_rayleigh", 
+		   rb_gsl_histogram_fit_rayleigh, -1);
   rb_define_method(cgsl_histogram, "fit", 
 		   rb_gsl_histogram_fit, -1);
 
@@ -1641,4 +1992,8 @@ void Init_gsl_histogram(VALUE module)
   rb_define_alias(cgsl_histogram, "mergebin", "rebin");
 
   rb_define_method(cgsl_histogram, "reverse", rb_gsl_histogram_reverse, 0);
+
+  rb_define_method(cgsl_histogram, "percentile", rb_gsl_histogram_percentile, 1);
+  rb_define_method(cgsl_histogram, "median", rb_gsl_histogram_median, 0);
+  rb_define_method(cgsl_histogram, "percentile_inv", rb_gsl_histogram_percentile_inv, 1);
 }
