@@ -79,7 +79,7 @@ static VALUE rb_gsl_isinf(VALUE obj, VALUE x)
 
 static VALUE rb_gsl_isinf2(VALUE obj, VALUE x)
 {
-  Need_Float(x);
+  //  Need_Float(x);
   if (gsl_isinf(NUM2DBL(x))) return Qtrue;
   else return Qfalse;
 }
@@ -153,12 +153,13 @@ static VALUE rb_gsl_math_eval(double (*func)(const double), VALUE xx)
     return rb_float_new((*func)(NUM2DBL(xx)));
     break;
   case T_ARRAY:
-    size = RARRAY(xx)->len;
+    size = RARRAY_LEN(xx);
     ary = rb_ary_new2(size);
     for (i = 0; i < size; i++) {
       x = rb_ary_entry(xx, i);
       Need_Float(x);
-      rb_ary_store(ary, i, rb_float_new((*func)(RFLOAT(x)->value)));
+      //      rb_ary_store(ary, i, rb_float_new((*func)(RFLOAT(x)->value)));
+      rb_ary_store(ary, i, rb_float_new((*func)(NUM2DBL(x))));
     }
     return ary;
     break;
@@ -210,14 +211,16 @@ static VALUE rb_gsl_math_eval2(double (*func)(const double, const double), VALUE
     break;
   case T_ARRAY:
     Check_Type(yy, T_ARRAY);
-    size = RARRAY(xx)->len;
-    if (size != RARRAY(yy)->len) rb_raise(rb_eRuntimeError, "array sizes are different.");
+    size = RARRAY_LEN(xx);
+    //    if (size != RARRAY(yy)->len) rb_raise(rb_eRuntimeError, "array sizes are different.");
+    if (size != RARRAY_LEN(yy)) rb_raise(rb_eRuntimeError, "array sizes are different.");
     ary = rb_ary_new2(size);
     for (i = 0; i < size; i++) {
       x = rb_ary_entry(xx, i);
       y = rb_ary_entry(yy, i);
       Need_Float(x); Need_Float(y);
-      rb_ary_store(ary, i, rb_float_new((*func)(RFLOAT(x)->value, RFLOAT(y)->value)));
+      //      rb_ary_store(ary, i, rb_float_new((*func)(RFLOAT(x)->value, RFLOAT(y)->value)));
+      rb_ary_store(ary, i, rb_float_new((*func)(NUM2DBL(x), NUM2DBL(y))));
     }
     return ary;
     break;
@@ -339,7 +342,7 @@ VALUE rb_gsl_pow(VALUE obj, VALUE xx, VALUE nn)
     break;
   case T_ARRAY:
     n = NUM2DBL(nn);
-    size = RARRAY(xx)->len;
+    size = RARRAY_LEN(xx);
     ary = rb_ary_new2(size);
     for (i = 0; i < size; i++) {
       x = rb_ary_entry(xx, i);
@@ -415,12 +418,13 @@ static VALUE rb_gsl_pow_int(VALUE obj, VALUE xx, VALUE nn)
   case T_ARRAY:
     CHECK_FIXNUM(nn);
     n = FIX2INT(nn);
-    size = RARRAY(xx)->len;
+    size = RARRAY_LEN(xx);
     ary = rb_ary_new2(size);
     for (i = 0; i < size; i++) {
       x = rb_ary_entry(xx, i);
       Need_Float(x);
-      rb_ary_store(ary, i, rb_float_new(gsl_pow_int(RFLOAT(x)->value, n)));
+      //      rb_ary_store(ary, i, rb_float_new(gsl_pow_int(RFLOAT(x)->value, n)));
+      rb_ary_store(ary, i, rb_float_new(gsl_pow_int(NUM2DBL(x), n)));
     }
     return ary;
     break;
