@@ -20,7 +20,15 @@ enum {
   GSL_MATRIX_COMPLEX_DIV,
 };
 
-static VALUE rb_gsl_matrix_complex_arithmetics(int flag, VALUE obj, VALUE bb);
+// From complex.c
+gsl_complex rb_gsl_obj_to_gsl_complex(VALUE obj, gsl_complex *z);
+
+// From ext/vector_source.c
+void get_range_beg_en_n(VALUE range, double *beg, double *en, size_t *n, int *step);
+
+// From ext/matrix_source.c
+void parse_submatrix_args(int argc, VALUE *argv, size_t size1, size_t size2,
+    size_t *i, size_t *j, size_t *n1, size_t *n2);
 
 static VALUE rb_gsl_matrix_complex_arithmetics(int flag, VALUE obj, VALUE bb)
 {
@@ -208,7 +216,8 @@ static VALUE rb_gsl_matrix_complex_eye(int argc, VALUE *argv, VALUE klass)
       z = gsl_complex_rect(NUM2DBL(argv[1]), 0.0);
       break;
     case T_ARRAY:
-      if (RARRAY(argv[1])->len < 2) rb_raise(rb_eArgError, "wrong argument");
+      //      if (RARRAY(argv[1])->len < 2) rb_raise(rb_eArgError, "wrong argument");
+      if (RARRAY_LEN(argv[1]) < 2) rb_raise(rb_eArgError, "wrong argument");
       z = gsl_complex_rect(NUM2DBL(rb_ary_entry(argv[1], 0)),
 			   NUM2DBL(rb_ary_entry(argv[1], 1)));
       break;
@@ -252,52 +261,145 @@ static VALUE rb_gsl_matrix_complex_identity(VALUE klass, VALUE nn)
   return  Data_Wrap_Struct(klass, 0, gsl_matrix_complex_free, m);
 }
 
-static VALUE rb_gsl_matrix_complex_set(int argc, VALUE *argv, VALUE obj)
+static VALUE rb_gsl_matrix_complex_set_zero(VALUE obj)
 {
   gsl_matrix_complex *m = NULL;
-  gsl_complex *z = NULL, tmp;
-  size_t i, j;
-  if (argc < 3) rb_raise(rb_eArgError, "wrong number of arguments");
-  CHECK_FIXNUM(argv[0]);
-  CHECK_FIXNUM(argv[1]);
   Data_Get_Struct(obj, gsl_matrix_complex, m);
-  i = FIX2INT(argv[0]);
-  j = FIX2INT(argv[1]);
-  switch (argc) {
-  case 3:
-    if (rb_obj_is_kind_of(argv[2], cgsl_complex)) {
-      Data_Get_Struct(argv[2], gsl_complex, z);
-    } else {
-      switch(TYPE(argv[2])) {
-      case T_ARRAY:
-	tmp = ary2complex(argv[2]);
-	z = &tmp;
-	break;
-      case T_FLOAT:
-      case T_BIGNUM:
-      case T_FIXNUM:
-	tmp.dat[0] = NUM2DBL(argv[2]);
-	tmp.dat[1] = 0.0;
-	z = &tmp;
-	break;
-      default:
-	rb_raise(rb_eTypeError, 
-		 "wrong argument type %s", rb_class2name(CLASS_OF(argv[2])));
-	break;
-      }
-      break;
-    }
+  gsl_matrix_complex_set_zero(m);
+  return obj;
+}
+
+static VALUE rb_gsl_matrix_complex_set_identity(VALUE obj)
+{
+  gsl_matrix_complex *m = NULL;
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  gsl_matrix_complex_set_identity(m);
+  return obj;
+}
+
+static VALUE rb_gsl_matrix_complex_set_all(VALUE obj, VALUE s)
+{
+  gsl_matrix_complex *m = NULL;
+  gsl_complex c, *z = NULL;
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  switch (TYPE(s)) {
+  case T_FLOAT:
+  case T_BIGNUM:
+  case T_FIXNUM:
+    c.dat[0] = NUM2DBL(s);
+    c.dat[1] = 0.0;
+    gsl_matrix_complex_set_all(m, c);
     break;
-  case 4:
-    tmp.dat[0] = NUM2DBL(argv[2]);
-    tmp.dat[1] = NUM2DBL(argv[3]);
-    z = &tmp;
+  case T_ARRAY:
+    c.dat[0] = NUM2DBL(rb_ary_entry(s, 0));
+    c.dat[1] = NUM2DBL(rb_ary_entry(s, 1));
+    gsl_matrix_complex_set_all(m, c);
     break;
   default:
-    rb_raise(rb_eArgError, "wrong number of arguments");
+    if (rb_obj_is_kind_of(s, cgsl_complex)) {
+      Data_Get_Struct(s, gsl_complex, z);
+      gsl_matrix_complex_set_all(m, *z);
+    } else {
+      rb_raise(rb_eTypeError, 
+	       "wrong argument type %s", rb_class2name(CLASS_OF(s)));
+    }
     break;
   }
-  gsl_matrix_complex_set(m, i, j, *z);
+  return obj;
+}
+
+void rb_gsl_vector_complex_set_subvector(int argc, VALUE *argv, gsl_vector_complex *v, VALUE other);
+static VALUE rb_gsl_matrix_complex_set(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_matrix_complex *m = NULL, *mother;
+  gsl_matrix_complex_view mv;
+  gsl_vector_complex_view vv;
+  gsl_complex tmp;
+  VALUE other, row, row_set_argv[2];
+  int ii, ij, step;
+  size_t i, j, k, n1, n2, nother;
+  double beg, end;
+
+  if(argc < 1 || argc > 5) {
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 1-5)", argc);
+  }
+
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  other = argv[argc-1];
+
+  if(argc == 1) {
+    // m[] = x
+    gsl_matrix_complex_set_all(m, rb_gsl_obj_to_gsl_complex(other, NULL));
+  } else if(argc == 3 && TYPE(argv[0]) == T_FIXNUM && TYPE(argv[1]) == T_FIXNUM) {
+    // m[i,j] = x
+    ii = FIX2INT(argv[0]);
+    ij = FIX2INT(argv[1]);
+    if(ii < 0) ii += m->size1;
+    if(ij < 0) ij += m->size2;
+    gsl_matrix_complex_set(m, (size_t)ii, (size_t)ij, rb_gsl_obj_to_gsl_complex(other, NULL));
+  } else if(TYPE(argv[0]) == T_ARRAY) {
+    // m.set(row0...)
+    row_set_argv[0] = INT2FIX(0);
+    row_set_argv[1] = INT2FIX(m->size2);
+
+    for(k = 0; k < argc && k < m->size1; k++) {
+      vv = gsl_matrix_complex_row(m, k);
+      rb_gsl_vector_complex_set_subvector(2, row_set_argv, &vv.vector, argv[k]);
+    }
+  } else {
+    // x -> assignment to m.submatrix(i...)
+    parse_submatrix_args(argc-1, argv, m->size1, m->size2, &i, &j, &n1, &n2);
+    if(n1 == 0) n1 = 1;
+    if(n2 == 0) n2 = 1;
+    mv = gsl_matrix_complex_submatrix(m, i, j, n1, n2);
+    if(rb_obj_is_kind_of(other, cgsl_matrix_complex)) {
+      // m[...] = m_other
+      Data_Get_Struct(other, gsl_matrix_complex, mother);
+      if(n1 * n2 != mother->size1 * mother->size2) {
+        rb_raise(rb_eRangeError, "sizes do not match (%d x %d != %d x %d)",
+		 (int) n1, (int) n2, (int) mother->size1, (int) mother->size2);
+      }
+      // TODO Change to gsl_matrix_memmove if/when GSL has such a function
+      // because gsl_matrix_memcpy does not handle overlapping regions (e.g.
+      // Views) well.
+      gsl_matrix_complex_memcpy(&mv.matrix, mother);
+    } else if(rb_obj_is_kind_of(other, rb_cArray)) {
+      row_set_argv[0] = INT2FIX(0);
+      row_set_argv[1] = INT2FIX(n2);
+
+      if(n1 == 1) {
+        // m[...] = [col0, ...] # single row
+        vv = gsl_matrix_complex_row(&mv.matrix, 0);
+        rb_gsl_vector_complex_set_subvector(2, row_set_argv, &vv.vector, other);
+      } else {
+        // m[...] = [[row0], [row1], ...] # multiple rows
+        if(n1 != RARRAY_LEN(other)) {
+          rb_raise(rb_eRangeError, "row counts do not match (%d != %d)",
+		   (int) n1, (int) RARRAY_LEN(other));
+        }
+        for(k = 0; k < n1; k++) {
+          vv = gsl_matrix_complex_row(&mv.matrix, k);
+          row = rb_ary_entry(other, k);
+          rb_gsl_vector_complex_set_subvector(2, row_set_argv, &vv.vector, row);
+        }
+      }
+    } else if(rb_obj_is_kind_of(other, rb_cRange)) {
+      // m[...] = beg..end
+      get_range_beg_en_n(other, &beg, &end, &nother, &step);
+      if(n1 * n2 != nother) {
+        rb_raise(rb_eRangeError, "sizes do not match (%d x %d != %d)", (int) n1, (int) n2, (int) nother);
+      }
+      tmp = gsl_complex_rect(beg, 0.0);
+      for(k = 0; k < nother; k++) {
+        gsl_matrix_complex_set(&mv.matrix, k / n2, k % n2, tmp);
+        GSL_SET_REAL(&tmp, GSL_REAL(tmp) + step);
+      }
+    } else {
+      // m[...] = x
+      gsl_matrix_complex_set_all(&mv.matrix, rb_gsl_obj_to_gsl_complex(other, NULL));
+    }
+  }
+
   return obj;
 }
 
@@ -353,29 +455,105 @@ static VALUE rb_gsl_matrix_complex_set_col(int argc, VALUE *argv, VALUE obj)
   return obj;
 }
 
-static VALUE rb_gsl_matrix_complex_row(VALUE, VALUE);
-
+static VALUE rb_gsl_matrix_complex_submatrix(int argc, VALUE *argv, VALUE obj);
 static VALUE rb_gsl_matrix_complex_get(int argc, VALUE *argv, VALUE obj)
 {
   gsl_matrix_complex *m = NULL;
   gsl_complex *c = NULL;
-  size_t i, j;
-  switch (argc) {
-  case 1:
-    return rb_gsl_matrix_complex_row(obj, argv[0]);
-    break;
-  case 2:
-    CHECK_FIXNUM(argv[0]); CHECK_FIXNUM(argv[1]);
+  VALUE retval;
+  int ii, ij;
+
+  if(argc == 2 && TYPE(argv[0]) == T_FIXNUM && TYPE(argv[1]) == T_FIXNUM) {
+    // m[i,j]
     Data_Get_Struct(obj, gsl_matrix_complex, m);
-    i = FIX2INT(argv[0]); j = FIX2INT(argv[1]);
+    ii = FIX2INT(argv[0]);
+    ij = FIX2INT(argv[1]);
+    if(ii < 0) ii += m->size1;
+    if(ij < 0) ij += m->size2;
     c = ALLOC(gsl_complex);
-    *c = gsl_matrix_complex_get(m, i, j); 
-    return Data_Wrap_Struct(cgsl_complex, 0, free, c);
-    break;
-  default:
-    rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
-    break;
+    *c = gsl_matrix_complex_get(m, (size_t)ii, (size_t)ij); 
+    retval = Data_Wrap_Struct(cgsl_complex, 0, free, c);
+  } else if(argc == 1 && TYPE(argv[0]) == T_FIXNUM) {
+    // m[i]
+    Data_Get_Struct(obj, gsl_matrix_complex, m);
+    ii = FIX2INT(argv[0]);
+    if(ii < 0) ii += m->size1 * m->size2;
+    c = ALLOC(gsl_complex);
+    *c = gsl_matrix_complex_get(m, (size_t)(ii / m->size2), (size_t)(ii % m->size2));
+    retval = Data_Wrap_Struct(cgsl_complex, 0, free, c);
+  } else if(argc == 1 && TYPE(argv[0]) == T_ARRAY) {
+    // m[[i,j]], to have parity with Real Matrix types
+    if(RARRAY_LEN(argv[0]) == 2) {
+      Data_Get_Struct(obj, gsl_matrix_complex, m);
+      ii = FIX2INT(RARRAY_PTR(argv[0])[0]);
+      ij = FIX2INT(RARRAY_PTR(argv[0])[1]);
+      if(ii < 0) ii += m->size1;
+      if(ij < 0) ij += m->size2;
+      c = ALLOC(gsl_complex);
+      *c = gsl_matrix_complex_get(m, (size_t)ii, (size_t)ij); 
+      retval = Data_Wrap_Struct(cgsl_complex, 0, free, c);
+    } else {
+      rb_raise(rb_eArgError, "Array index must have length 2, not %d", (int) RARRAY_LEN(argv[0]));
+    }
+  } else {
+    retval = rb_gsl_matrix_complex_submatrix(argc, argv, obj);
   }
+
+  return retval;
+}
+
+static void rb_gsl_matrix_complex_collect_native(gsl_matrix_complex *src, gsl_matrix_complex *dst)
+{
+  VALUE vz;
+  gsl_complex * zp;
+  size_t i, j;
+  for (i = 0; i < src->size1; i++) {
+    for (j = 0; j < src->size2; j++) {
+      vz = Data_Make_Struct(cgsl_complex, gsl_complex, 0, free, zp);
+      *zp = gsl_matrix_complex_get(src, i, j);
+      vz = rb_yield(vz);
+      CHECK_COMPLEX(vz);
+      Data_Get_Struct(vz, gsl_complex, zp);
+      gsl_matrix_complex_set(dst, i, j, *zp);
+    }
+  }
+}
+
+static VALUE rb_gsl_matrix_complex_collect(VALUE obj)
+{
+  gsl_matrix_complex *m = NULL, *mnew;
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  mnew = gsl_matrix_complex_alloc(m->size1, m->size2);
+  rb_gsl_matrix_complex_collect_native(m, mnew);
+  return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, mnew);
+}
+
+static VALUE rb_gsl_matrix_complex_collect_bang(VALUE obj)
+{
+  gsl_matrix_complex *m = NULL;
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  rb_gsl_matrix_complex_collect_native(m, m);
+  return obj;
+}
+
+static VALUE rb_gsl_matrix_complex_to_a(VALUE obj)
+{
+  gsl_matrix_complex *m;
+  gsl_complex *c;
+  VALUE ma, ra;
+  size_t i, j;
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  ma = rb_ary_new2(m->size1);
+  for(i=0; i < m->size1; i++) {
+    ra = rb_ary_new2(m->size2);
+    rb_ary_store(ma, i, ra);
+    for(j=0; j < m->size2; j++) {
+      c = ALLOC(gsl_complex);
+      *c = gsl_matrix_complex_get(m, i, j); 
+      rb_ary_store(ra, j, Data_Wrap_Struct(cgsl_complex, 0, free, c));
+    }
+  }
+  return ma;
 }
 
 static VALUE rb_gsl_matrix_complex_ptr(VALUE obj, VALUE i, VALUE j)
@@ -420,6 +598,64 @@ static VALUE rb_gsl_matrix_complex_print(VALUE obj)
   return obj;
 }
 
+static VALUE rb_gsl_matrix_complex_to_s(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_matrix_complex *m = NULL;
+  char buf[64];
+  size_t i, j;
+  VALUE str;
+  gsl_complex z;
+  int max_rows = 4;
+  int max_cols = 4;
+
+  switch(argc){
+    case 2: max_cols = NUM2INT(argv[1]);
+    case 1: max_rows = NUM2INT(argv[0]);
+    case 0: break;
+    default:
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0, 1, or 2)", argc);
+  }
+
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  if (m->size1 == 0 && m->size2 == 0) return rb_str_new2("[ ]");
+  str = rb_str_new2("[ ");
+  for (i = 0; i < m->size1; i++) {
+    if (i != 0) {
+      rb_str_cat(str, "\n  ", 3);
+    }
+    for (j = 0; j < m->size2; j++) {
+      z = gsl_matrix_complex_get(m, i, j);
+      sprintf(buf,
+          "%s[ %4.3e %4.3e ]", (j==0) ? "" : " ", GSL_REAL(z), GSL_IMAG(z));
+      rb_str_cat(str, buf, strlen(buf));
+      // if too many cols
+      if (j >= max_cols-1 && j != m->size2-1) {
+        rb_str_cat(str, " ...", 4);
+        break;
+      }
+    }
+    // if too many rows
+    if (i >= max_rows-1 && i != m->size1-1) {
+      rb_str_cat(str, "\n  ...", 6);
+      break;
+    }
+  }
+  rb_str_cat(str, " ]", 2);
+  return str;
+}
+
+static VALUE rb_gsl_matrix_complex_inspect(int argc, VALUE *argv, VALUE obj)
+{
+  VALUE str;
+  char buf[128];
+  gsl_matrix_complex *m;
+
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  sprintf(buf, "#<%s[%lu,%lu]:%#x>\n", rb_class2name(CLASS_OF(obj)), m->size1, m->size2, NUM2UINT(rb_obj_id(obj)));
+  str = rb_str_new2(buf);
+  return rb_str_concat(str, rb_gsl_matrix_complex_to_s(argc, argv, obj));
+}
+
 static VALUE rb_gsl_matrix_complex_fwrite(VALUE obj, VALUE io)
 {
   gsl_matrix_complex *h = NULL;
@@ -696,77 +932,28 @@ static VALUE rb_gsl_matrix_complex_add_diagonal(VALUE obj, VALUE s)
   return obj;
 }
 
-static VALUE rb_gsl_matrix_complex_set_zero(VALUE obj)
-{
-  gsl_matrix_complex *m = NULL;
-  Data_Get_Struct(obj, gsl_matrix_complex, m);
-  gsl_matrix_complex_set_zero(m);
-  return obj;
-}
-
-static VALUE rb_gsl_matrix_complex_set_identity(VALUE obj)
-{
-  gsl_matrix_complex *m = NULL;
-  Data_Get_Struct(obj, gsl_matrix_complex, m);
-  gsl_matrix_complex_set_identity(m);
-  return obj;
-}
-
-static VALUE rb_gsl_matrix_complex_set_all(VALUE obj, VALUE s)
-{
-  gsl_matrix_complex *m = NULL;
-  gsl_complex c, *z = NULL;
-  Data_Get_Struct(obj, gsl_matrix_complex, m);
-  switch (TYPE(s)) {
-  case T_FLOAT:
-  case T_BIGNUM:
-  case T_FIXNUM:
-    c.dat[0] = NUM2DBL(s);
-    c.dat[1] = 0.0;
-    gsl_matrix_complex_set_all(m, c);
-    break;
-  case T_ARRAY:
-    c.dat[0] = NUM2DBL(rb_ary_entry(s, 0));
-    c.dat[1] = NUM2DBL(rb_ary_entry(s, 1));
-    gsl_matrix_complex_set_all(m, c);
-    break;
-  default:
-    if (rb_obj_is_kind_of(s, cgsl_complex)) {
-      Data_Get_Struct(s, gsl_complex, z);
-      gsl_matrix_complex_set_all(m, *z);
-    } else {
-      rb_raise(rb_eTypeError, 
-	       "wrong argument type %s", rb_class2name(CLASS_OF(s)));
-    }
-    break;
-  }
-  return obj;
-}
-
 static VALUE rb_gsl_matrix_complex_submatrix(int argc, VALUE *argv, VALUE obj)
 {
   gsl_matrix_complex *m = NULL;
   gsl_matrix_complex_view *mv = NULL;
+  gsl_vector_complex_view *vv = NULL;
   size_t i, j, n1, n2;
   Data_Get_Struct(obj, gsl_matrix_complex, m);
-  switch (argc) {
-  case 0:
-    i = 0; j = 0;
-    n1 = m->size1; n2 = m->size2;
-    break;
-  case 4:
-    CHECK_FIXNUM(argv[0]); CHECK_FIXNUM(argv[1]);
-    CHECK_FIXNUM(argv[2]); CHECK_FIXNUM(argv[3]);
-    i = FIX2INT(argv[0]);  j = FIX2INT(argv[1]);
-    n1 = FIX2INT(argv[2]);  n2 = FIX2INT(argv[3]);
-    break;
-  default:
-    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 4)", argc);
-    break;
+  parse_submatrix_args(argc, argv, m->size1, m->size2, &i, &j, &n1, &n2);
+  if(n1 == 0) {
+    vv = ALLOC(gsl_vector_complex_view);
+    *vv = gsl_matrix_complex_subrow(m, i, j, n2);
+    return Data_Wrap_Struct(cgsl_vector_complex_view, 0, free, vv);
+  }
+  else if(n2 == 0) {
+    vv = ALLOC(gsl_vector_complex_view);
+    *vv = gsl_matrix_complex_subcolumn(m, j, i, n1);
+    return Data_Wrap_Struct(cgsl_vector_complex_col_view, 0, free, vv);
+  } else {
+    mv = ALLOC(gsl_matrix_complex_view);
+    *mv = gsl_matrix_complex_submatrix(m, i, j, n1, n2);
+    return Data_Wrap_Struct(cgsl_matrix_complex_view, 0, free, mv);
   }
-  mv = gsl_matrix_complex_view_alloc();
-  *mv = gsl_matrix_complex_submatrix(m, i, j, n1, n2);
-  return Data_Wrap_Struct(cgsl_matrix_complex_view, 0, gsl_matrix_complex_view_free, mv);
 }
 
 static VALUE rb_gsl_matrix_complex_row(VALUE obj, VALUE i)
@@ -986,27 +1173,29 @@ static VALUE rb_gsl_matrix_complex_trace(VALUE obj)
 static VALUE rb_gsl_matrix_complex_each_row(VALUE obj)
 {
   gsl_matrix_complex *m = NULL;
-  gsl_vector_complex_view vv, *pv = &vv;
+  gsl_vector_complex_view *vv;
   size_t i;
   Data_Get_Struct(obj, gsl_matrix_complex, m);
   for (i = 0; i < m->size1; i++) {
-    vv = gsl_matrix_complex_row(m, i);
-    rb_yield(Data_Wrap_Struct(cgsl_vector_complex_view_ro, 0, NULL, pv));
+    vv = ALLOC(gsl_vector_complex_view);
+    *vv = gsl_matrix_complex_row(m, i);
+    rb_yield(Data_Wrap_Struct(cgsl_vector_complex_view, 0, free, vv));
   }
-  return Qtrue;
+  return obj;
 }
 
 static VALUE rb_gsl_matrix_complex_each_col(VALUE obj)
 {
   gsl_matrix_complex *m = NULL;
-  gsl_vector_complex_view vv, *pv = &vv;
+  gsl_vector_complex_view *vv;
   size_t i;
   Data_Get_Struct(obj, gsl_matrix_complex, m);
   for (i = 0; i < m->size2; i++) {
-    vv = gsl_matrix_complex_column(m, i);
-    rb_yield(Data_Wrap_Struct(cgsl_vector_complex_view_ro, 0, NULL, pv));
+    vv = ALLOC(gsl_vector_complex_view);
+    *vv = gsl_matrix_complex_column(m, i);
+    rb_yield(Data_Wrap_Struct(cgsl_vector_complex_col_view, 0, free, vv));
   }
-  return Qtrue;
+  return obj;
 }
 
 static VALUE rb_gsl_matrix_complex_size1(VALUE obj)
@@ -1021,6 +1210,12 @@ static VALUE rb_gsl_matrix_complex_size2(VALUE obj)
   Data_Get_Struct(obj, gsl_matrix_complex, m);
   return INT2FIX(m->size2);
 }
+static VALUE rb_gsl_matrix_complex_shape(VALUE obj)
+{
+  gsl_matrix_complex *m = NULL;
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  return rb_ary_new3(2, INT2FIX(m->size1), INT2FIX(m->size2));
+}
 
 static VALUE rb_gsl_matrix_complex_uplus(VALUE obj)
 {
@@ -1232,84 +1427,96 @@ static VALUE rb_gsl_matrix_complex_arccoth(VALUE obj)
 
 static VALUE rb_gsl_matrix_complex_indgen_bang(int argc, VALUE *argv[], VALUE obj)
 {
-  gsl_matrix_complex *v = NULL;
-  size_t start = 0, step = 1;
+  gsl_matrix_complex *m = NULL;
+  double start = 0, step = 1, x;
   size_t i, j;
   switch (argc) {
   case 0:
     break;
   case 1:
-    start = FIX2INT(argv[0]);
+    start = NUM2DBL(argv[0]);
     break;
   case 2:
-    start = FIX2INT(argv[0]);
-    step = FIX2INT(argv[1]);
+    start = NUM2DBL(argv[0]);
+    step = NUM2DBL(argv[1]);
     break;
   default:
-    rb_raise(rb_eArgError, "Wrong number of arguments (%d for 0-2)");
+    rb_raise(rb_eArgError, "Wrong number of arguments (%d for 0-2)", argc);
   }
-  Data_Get_Struct(obj, gsl_matrix_complex, v);
-  for (i = 0, j = start; i < v->block->size; i++, j += step) {
-    v->data[2*i] = (double) j;
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  x = start;
+  for (i = 0; i < m->size1; i++) {
+    for (j = 0; j < m->size2; j++) {
+      gsl_matrix_complex_set(m, i, j, gsl_complex_rect(x, 0));
+      x += step;
+    }
   }
   return obj;
 }
 
 static VALUE rb_gsl_matrix_complex_indgen(int argc, VALUE *argv, VALUE obj)
 {
-  gsl_matrix_complex *v = NULL, *vnew;
-  size_t start = 0, step = 1;
+  gsl_matrix_complex *m = NULL, *mnew;
+  double start = 0, step = 1, x;
   size_t i, j;
   switch (argc) {
   case 0:
     break;
   case 1:
-    start = FIX2INT(argv[0]);
+    start = NUM2DBL(argv[0]);
     break;
   case 2:
-    start = FIX2INT(argv[0]);
-    step = FIX2INT(argv[1]);
+    start = NUM2DBL(argv[0]);
+    step = NUM2DBL(argv[1]);
     break;
   default:
-    rb_raise(rb_eArgError, "Wrong number of arguments (%d for 0-2)");
+    rb_raise(rb_eArgError, "Wrong number of arguments (%d for 0-2)", argc);
   }
-  Data_Get_Struct(obj, gsl_matrix_complex, v);
-  vnew = gsl_matrix_complex_calloc(v->size1, v->size2);
-  for (i = 0, j = start; i < vnew->block->size; i++, j += step) {
-    vnew->data[2*i] = (double) j;
+  Data_Get_Struct(obj, gsl_matrix_complex, m);
+  mnew = gsl_matrix_complex_calloc(m->size1, m->size2);
+  x = start;
+  for (i = 0; i < mnew->size1; i++) {
+    for (j = 0; j < mnew->size2; j++) {
+      gsl_matrix_complex_set(mnew, i, j, gsl_complex_rect(x, 0));
+      x += step;
+    }
   }
-  return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, vnew);
+  return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, mnew);
 }
 
 static VALUE rb_gsl_matrix_complex_indgen_singleton(int argc, VALUE *argv, VALUE obj)
 {
-  gsl_matrix_complex *vnew;
-  size_t n1, n2, start = 0, step = 1;
-  size_t i, j;
+  gsl_matrix_complex *mnew;
+  double start = 0, step = 1, x;
+  size_t n1, n2, i, j;
   switch (argc) {
   case 2:
-    n1 = (size_t) FIX2INT(argv[0]);
-    n2 = (size_t) FIX2INT(argv[1]);
+    n1 = (size_t) NUM2INT(argv[0]);
+    n2 = (size_t) NUM2INT(argv[1]);
     break;
   case 3:
-    n1 = (size_t) FIX2INT(argv[0]);
-    n2 = (size_t) FIX2INT(argv[1]);
-    start = FIX2INT(argv[2]);
+    n1 = (size_t) NUM2INT(argv[0]);
+    n2 = (size_t) NUM2INT(argv[1]);
+    start = NUM2DBL(argv[2]);
     break;
   case 4:
-    n1 = (size_t) FIX2INT(argv[0]);
-    n2 = (size_t) FIX2INT(argv[1]);
-    start = FIX2INT(argv[2]);
-    step = FIX2INT(argv[3]);
+    n1 = (size_t) NUM2INT(argv[0]);
+    n2 = (size_t) NUM2INT(argv[1]);
+    start = NUM2DBL(argv[2]);
+    step = NUM2DBL(argv[3]);
     break;
   default:
-    rb_raise(rb_eArgError, "Wrong number of arguments (%d for 0-4)");
+    rb_raise(rb_eArgError, "Wrong number of arguments (%d for 0-4)", argc);
   }
-  vnew = gsl_matrix_complex_calloc(n1, n2);
-  for (i = 0, j = start; i < vnew->block->size; i++, j += step) {
-    vnew->data[2*i] = (double) j;
+  mnew = gsl_matrix_complex_calloc(n1, n2);
+  x = start;
+  for (i = 0; i < mnew->size1; i++) {
+    for (j = 0; j < mnew->size2; j++) {
+      gsl_matrix_complex_set(mnew, i, j, gsl_complex_rect(x, 0));
+      x += step;
+    }
   }
-  return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, vnew);
+  return Data_Wrap_Struct(cgsl_matrix_complex, 0, gsl_matrix_complex_free, mnew);
 }
 
 
@@ -1374,6 +1581,7 @@ void Init_gsl_matrix_complex(VALUE module)
   rb_define_singleton_method(cgsl_matrix_complex, "I", rb_gsl_matrix_complex_identity, 1);
 
   rb_define_method(cgsl_matrix_complex, "set", rb_gsl_matrix_complex_set, -1);
+  rb_define_alias(cgsl_matrix_complex, "[]=", "set");
 
   rb_define_method(cgsl_matrix_complex, "set_row", rb_gsl_matrix_complex_set_row, -1);
   rb_define_method(cgsl_matrix_complex, "set_col", rb_gsl_matrix_complex_set_col, -1);
@@ -1382,10 +1590,11 @@ void Init_gsl_matrix_complex(VALUE module)
   rb_define_alias(cgsl_matrix_complex, "[]", "get");
   rb_define_method(cgsl_matrix_complex, "ptr", rb_gsl_matrix_complex_ptr, 2);
 
+  rb_define_method(cgsl_matrix_complex, "to_s", rb_gsl_matrix_complex_to_s, -1);
   rb_define_method(cgsl_matrix_complex, "fprintf", rb_gsl_matrix_complex_fprintf, -1);
   rb_define_method(cgsl_matrix_complex, "printf", rb_gsl_matrix_complex_printf, -1);
   rb_define_method(cgsl_matrix_complex, "print", rb_gsl_matrix_complex_print, 0);
-  rb_define_method(cgsl_matrix_complex, "inspect", rb_gsl_matrix_complex_print, 0);
+  rb_define_method(cgsl_matrix_complex, "inspect", rb_gsl_matrix_complex_inspect, -1);
   rb_define_method(cgsl_matrix_complex, "fwrite", rb_gsl_matrix_complex_fwrite, 1);
   rb_define_method(cgsl_matrix_complex, "fread", rb_gsl_matrix_complex_fread, 1);
   rb_define_method(cgsl_matrix_complex, "fscanf", rb_gsl_matrix_complex_fscanf, 1);
@@ -1393,6 +1602,7 @@ void Init_gsl_matrix_complex(VALUE module)
   rb_define_singleton_method(cgsl_matrix_complex, "memcpy", rb_gsl_matrix_complex_memcpy, 2);
   rb_define_method(cgsl_matrix_complex, "clone", rb_gsl_matrix_complex_clone, 0);
   rb_define_alias(cgsl_matrix_complex, "duplicate", "clone");
+  rb_define_alias(cgsl_matrix_complex, "dup", "clone");
   rb_define_method(cgsl_matrix_complex, "swap_rows", rb_gsl_matrix_complex_swap_rows, 2);
   rb_define_method(cgsl_matrix_complex, "swap_columns", rb_gsl_matrix_complex_swap_columns, 2);
   rb_define_method(cgsl_matrix_complex, "swap_rowcol", rb_gsl_matrix_complex_swap_rowcol, 2);
@@ -1424,6 +1634,7 @@ void Init_gsl_matrix_complex(VALUE module)
   rb_define_method(cgsl_matrix_complex, "column", rb_gsl_matrix_complex_column, 1);
   rb_define_alias(cgsl_matrix_complex, "col", "column");
   rb_define_method(cgsl_matrix_complex, "diagonal", rb_gsl_matrix_complex_diagonal, 0);
+  rb_define_alias(cgsl_matrix_complex, "diag", "diagonal");
   rb_define_method(cgsl_matrix_complex, "set_diagonal", rb_gsl_matrix_complex_set_diagonal, 1);
   rb_define_method(cgsl_matrix_complex, "subdiagonal", rb_gsl_matrix_complex_subdiagonal, 1);
   rb_define_method(cgsl_matrix_complex, "superdiagonal", rb_gsl_matrix_complex_superdiagonal, 1);
@@ -1442,7 +1653,9 @@ void Init_gsl_matrix_complex(VALUE module)
 
   /* 25.June.2004 */
   rb_define_method(cgsl_matrix_complex, "conjugate!", rb_gsl_matrix_complex_conjugate, 0);
+  rb_define_alias(cgsl_matrix_complex, "conj!", "conjugate!");
   rb_define_method(cgsl_matrix_complex, "conjugate", rb_gsl_matrix_complex_conjugate2, 0);
+  rb_define_alias(cgsl_matrix_complex, "conj", "conjugate");
   rb_define_method(cgsl_matrix_complex, "dagger!", rb_gsl_matrix_complex_dagger, 0);
   rb_define_method(cgsl_matrix_complex, "dagger", rb_gsl_matrix_complex_dagger2, 0);
   
@@ -1450,9 +1663,17 @@ void Init_gsl_matrix_complex(VALUE module)
   rb_define_method(cgsl_matrix_complex, "each_row", rb_gsl_matrix_complex_each_row, 0);
   rb_define_method(cgsl_matrix_complex, "each_col", rb_gsl_matrix_complex_each_col, 0);
   rb_define_alias(cgsl_matrix_complex, "each_column", "each_col");
+  rb_define_method(cgsl_matrix_complex, "collect", rb_gsl_matrix_complex_collect, 0);
+  rb_define_method(cgsl_matrix_complex, "collect!", rb_gsl_matrix_complex_collect_bang, 0);
+  rb_define_alias(cgsl_matrix_complex, "map", "collect");
+  rb_define_alias(cgsl_matrix_complex, "map!", "collect!");
   
+  rb_define_method(cgsl_matrix_complex, "to_a", rb_gsl_matrix_complex_to_a, 0);
+
   rb_define_method(cgsl_matrix_complex, "size1", rb_gsl_matrix_complex_size1, 0);
   rb_define_method(cgsl_matrix_complex, "size2", rb_gsl_matrix_complex_size2, 0);
+  rb_define_method(cgsl_matrix_complex, "shape", rb_gsl_matrix_complex_shape, 0);
+  rb_define_alias(cgsl_matrix_complex, "size", "shape");
   
   /*****/
   rb_define_method(cgsl_matrix_complex, "-@", rb_gsl_matrix_complex_uminus, 0);
@@ -1460,6 +1681,7 @@ void Init_gsl_matrix_complex(VALUE module)
 
   /****/
   rb_define_method(cgsl_matrix_complex, "arg", rb_gsl_matrix_complex_arg, 0);
+  rb_define_alias(cgsl_matrix_complex, "angle", "arg");
   rb_define_alias(cgsl_matrix_complex, "phase", "arg");
   rb_define_method(cgsl_matrix_complex, "abs", rb_gsl_matrix_complex_abs, 0);
   rb_define_alias(cgsl_matrix_complex, "amp", "abs");
