@@ -13,6 +13,11 @@
 #include "rb_gsl_function.h"
 #include "rb_gsl_common.h"
 
+#ifdef HAVE_NDLINEAR_GSL_MULTIFIT_NDLINEAR_H
+#include <ndlinear/gsl_multifit_ndlinear.h>
+void Init_ndlinear(VALUE module);
+#endif
+
 #ifndef CHECK_WORKSPACE
 #define CHECK_WORKSPACE(x) if(CLASS_OF(x)!=cgsl_multifit_workspace)\
       rb_raise(rb_eTypeError,\
@@ -40,7 +45,7 @@ static VALUE rb_gsl_multifit_workspace_new(VALUE klass, VALUE n, VALUE p)
   gsl_multifit_linear_workspace *w = NULL;
   CHECK_FIXNUM(n); CHECK_FIXNUM(p);
   w = gsl_multifit_linear_alloc(FIX2INT(n), FIX2INT(p));
-  return Data_Wrap_Struct(klass, 0, gsl_multifit_linear_free, w);
+  return Data_Wrap_Struct(cgsl_multifit_workspace, 0, gsl_multifit_linear_free, w);
 }
 
 static VALUE rb_gsl_multifit_linear(int argc, VALUE *argv, VALUE obj)
@@ -231,7 +236,7 @@ static VALUE rb_gsl_multifit_fdfsolver_new(int argc, VALUE *argv, VALUE klass)
       }
       break;
     default:
-      rb_raise(rb_eTypeError, "wrong argument type (Fixnum or String)",
+      rb_raise(rb_eTypeError, "wrong argument type %s (Fixnum or String)",
 	       rb_class2name(CLASS_OF(argv[0])));
       break;
     }
@@ -621,7 +626,7 @@ static int gsl_multifit_function_fdf_f(const gsl_vector *x, void *params,
   proc = rb_ary_entry(ary, 0);
   vx = Data_Wrap_Struct(cgsl_vector, 0, NULL, (gsl_vector *) x);
   vf = Data_Wrap_Struct(cgsl_vector, 0, NULL, f);
-  switch (RARRAY(vt_y_sigma)->len) {
+  switch (RARRAY_LEN(vt_y_sigma)) {
   case 2:
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
@@ -649,7 +654,7 @@ static int gsl_multifit_function_fdf_df(const gsl_vector *x, void *params,
   proc = rb_ary_entry(ary, 1);
   vx = Data_Wrap_Struct(cgsl_vector, 0, NULL, (gsl_vector *) x);
   vJ = Data_Wrap_Struct(cgsl_matrix, 0, NULL, J);
-  switch (RARRAY(vt_y_sigma)->len) {
+  switch (RARRAY_LEN(vt_y_sigma)) {
   case 2:
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
@@ -681,7 +686,7 @@ static int gsl_multifit_function_fdf_fdf(const gsl_vector *x, void *params,
   vx = Data_Wrap_Struct(cgsl_vector, 0, NULL, (gsl_vector *) x);
   vf = Data_Wrap_Struct(cgsl_vector, 0, NULL, f);
   vJ = Data_Wrap_Struct(cgsl_matrix, 0, NULL, J);
-  switch (RARRAY(vt_y_sigma)->len) {
+  switch (RARRAY_LEN(vt_y_sigma)) {
   case 2:
     vt = rb_ary_entry(vt_y_sigma, 0);
     vy = rb_ary_entry(vt_y_sigma, 1);
@@ -1412,6 +1417,64 @@ static int Lognormal_fdf(const gsl_vector *v, void *data,
   return GSL_SUCCESS;
 }
 
+/* Rayleigh fit 
+   y = A exp(-x*x/2/var)
+       v[0] = A
+       v[1] = var = sigma^2
+*/
+static int Rayleigh_f(const gsl_vector *v, void *data, gsl_vector *f)
+{
+  gsl_vector *x, *y, *w;
+  double A, var, xi, yi, wi;
+  size_t i;
+  struct fitting_xydata *xydata;
+  xydata = (struct fitting_xydata*) data;
+  x = xydata->x;
+  y = xydata->y;
+  w = xydata->w;
+  var = gsl_vector_get(v, 1);
+  A = gsl_vector_get(v, 0);
+  for (i = 0; i < x->size; i++) {
+    xi = gsl_vector_get(x, i);
+    yi = gsl_vector_get(y, i);
+    if (w) wi = gsl_vector_get(w, i);
+    else wi = 1.0;
+    gsl_vector_set(f, i, (A*xi*exp(-xi*xi/var/2.0) - yi)*wi);
+  }
+  return GSL_SUCCESS;
+}
+
+static int Rayleigh_df(const gsl_vector *v, void *data, gsl_matrix *J)
+{
+  double A, var, xi, yy, wi;
+  size_t i;
+  struct fitting_xydata *xydata;
+  gsl_vector *x, *y, *w;
+  xydata = (struct fitting_xydata*) data;
+  x = xydata->x;
+  y = xydata->y;
+  w = xydata->w;
+  var = gsl_vector_get(v, 1);
+  A = gsl_vector_get(v, 0);
+  for (i = 0; i < x->size; i++) {
+    xi = gsl_vector_get(x, i);
+    if (w) wi = gsl_vector_get(w, i);
+    else wi = 1.0;
+    yy = xi*exp(-xi*xi/var/2.0);
+    gsl_matrix_set(J, i, 1, A*yy*xi*xi/2/var/var*wi);
+    gsl_matrix_set(J, i, 0, yy*wi);
+  }
+  return GSL_SUCCESS;
+}
+
+static int Rayleigh_fdf(const gsl_vector *v, void *data,
+			gsl_vector *f, gsl_matrix *J)
+{
+  Rayleigh_f(v, data, f);
+  Rayleigh_df(v, data, J);
+  return GSL_SUCCESS;
+}
+
 static void set_fittype(gsl_multifit_function_fdf *f, const char *fittype, size_t *p, gsl_vector **v, int *flag)
 {
   if (str_tail_grep(fittype, "aussian_2peaks") == 0) {
@@ -1455,6 +1518,17 @@ static void set_fittype(gsl_multifit_function_fdf *f, const char *fittype, size_
       gsl_vector_set(*v, 2, 1);   
       *flag = 1;
     }
+ } else if (str_head_grep(fittype, "rayleigh") == 0) {
+    f->f = Rayleigh_f;
+    f->df = Rayleigh_df;
+    f->fdf = Rayleigh_fdf;
+    *p = 2;
+    if (*v == NULL) {
+      *v = gsl_vector_alloc(*p);
+      gsl_vector_set(*v, 0, 1);   /* A = 1 */
+      gsl_vector_set(*v, 1, 1);   /* sigma = 1 */
+      *flag = 1;
+    }
   } else if (str_head_grep(fittype, "dblexp") == 0) {
     f->f = DblExponential_f;
     f->df = DblExponential_df;
@@ -1660,6 +1734,34 @@ static VALUE rb_gsl_multifit_linear_est(VALUE module, VALUE xx, VALUE cc, VALUE
   gsl_multifit_linear_est(x, c, cov, &y, &y_err);
   return rb_ary_new3(2, rb_float_new(y), rb_float_new(y_err));
 }
+#endif
+#ifdef GSL_1_11_LATER
+static VALUE rb_gsl_multifit_linear_residuals(int argc, VALUE argv[], VALUE module)
+{
+  gsl_vector *y, *c, *r;
+  gsl_matrix *X;
+  VALUE ret;
+  switch (argc) {
+  case 3:
+    Data_Get_Matrix(argv[0], X);
+    Data_Get_Vector(argv[1], y);
+    Data_Get_Vector(argv[2], c);
+    r = gsl_vector_alloc(y->size);
+    ret = Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, r);
+    break;
+  case 4:
+    Data_Get_Matrix(argv[0], X);
+    Data_Get_Vector(argv[1], y);
+    Data_Get_Vector(argv[2], c);
+    Data_Get_Vector(argv[3], r);
+    ret = argv[3];
+    break;
+  default:
+    rb_raise(rb_eArgError, "Wrong number of arguments %d (3 or 4).\n", argc);
+  }
+  gsl_multifit_linear_residuals(X, y, c, r);
+  return ret;
+}
 
 #endif
 
@@ -1677,6 +1779,8 @@ void Init_gsl_multifit(VALUE module)
 			     rb_gsl_multifit_workspace_new, 2);
   rb_define_singleton_method(cgsl_multifit_workspace, "alloc", 
 			     rb_gsl_multifit_workspace_new, 2);
+  rb_define_singleton_method(mgsl_multifit, "alloc", 
+			     rb_gsl_multifit_workspace_new, 2);			     
   rb_define_module_function(mgsl_multifit, "linear", rb_gsl_multifit_linear, -1);
   rb_define_module_function(mgsl_multifit, "wlinear", rb_gsl_multifit_wlinear, -1);
 
@@ -1704,7 +1808,7 @@ void Init_gsl_multifit(VALUE module)
   rb_define_method(cgsl_multifit_fdfsolver, "name", rb_gsl_multifit_fdfsolver_name, 0);
   rb_define_method(cgsl_multifit_fdfsolver, "iterate", rb_gsl_multifit_fdfsolver_iterate, 0);
   rb_define_method(cgsl_multifit_fdfsolver, "position", rb_gsl_multifit_fdfsolver_position, 0);
-  rb_define_alias(cgsl_multifit_fdfsolver, "x", "position");
+  //  rb_define_alias(cgsl_multifit_fdfsolver, "x", "position");
   rb_define_method(cgsl_multifit_fdfsolver, "print_state", rb_gsl_multifit_fdfsolver_print_state, 1);
   rb_define_method(cgsl_multifit_fdfsolver, "fdf", rb_gsl_multifit_fdfsolver_fdf, 0);
   rb_define_method(cgsl_multifit_fdfsolver, "test_delta", rb_gsl_multifit_fdfsolver_test_delta, 2);
@@ -1754,6 +1858,15 @@ void Init_gsl_multifit(VALUE module)
   /***/
 #ifdef GSL_1_8_LATER
   rb_define_module_function(mgsl_multifit, "linear_est", rb_gsl_multifit_linear_est, 3);
+  rb_define_module_function(module, "multifit_linear_est", rb_gsl_multifit_linear_est, 3);
+#endif
+#ifdef GSL_1_11_LATER
+  rb_define_module_function(mgsl_multifit, "linear_residuals", rb_gsl_multifit_linear_residuals, -1);
+  rb_define_module_function(module, "multifit_linear_residuals", rb_gsl_multifit_linear_residuals, -1);
+#endif
+
+#ifdef HAVE_NDLINEAR_GSL_MULTIFIT_NDLINEAR_H
+  Init_ndlinear(mgsl_multifit);
 #endif
 
 }
