@@ -70,6 +70,9 @@ double gsl_poly_int_eval(const BASE c[], const int len, const double x)
 #ifdef HAVE_NARRAY_H
 #include "rb_gsl_with_narray.h"
 #endif
+#ifdef GSL_1_11_LATER
+static VALUE rb_gsl_complex_poly_complex_eval(VALUE a, VALUE b);
+#endif
 static VALUE rb_gsl_poly_eval_singleton(VALUE klass, VALUE a, VALUE x)
 {
   gsl_vector *v = NULL, *vx, *vnew;
@@ -83,8 +86,18 @@ static VALUE rb_gsl_poly_eval_singleton(VALUE klass, VALUE a, VALUE x)
 #ifdef HAVE_NARRAY_H
   int shape[1];
 #endif  
+#ifdef GSL_1_11_LATER
+  gsl_complex *z, zz;
+  gsl_vector_complex *vz, *vznew;
+  if (rb_obj_is_kind_of(a, cgsl_vector_complex)) 
+    return rb_gsl_complex_poly_complex_eval(a, x);
+#endif
   switch (TYPE(a)) {
   case T_ARRAY:
+#ifdef GSL_1_11_LATER
+    if (rb_obj_is_kind_of(rb_ary_entry(a, 0), cgsl_complex)) 
+      return rb_gsl_complex_poly_complex_eval(a, x);
+#endif
     v = make_cvector_from_rarray(a);
     N = v->size;
     ptr0 = v->data;
@@ -112,7 +125,7 @@ static VALUE rb_gsl_poly_eval_singleton(VALUE klass, VALUE a, VALUE x)
     val = rb_float_new(gsl_poly_eval(ptr0, N, NUM2DBL(x)));
     break;    
   case T_ARRAY:
-    n = RARRAY(x)->len;
+    n = RARRAY_LEN(x);
     val = rb_ary_new2(n);
     for (i = 0; i < n; i++) {
       rslt = gsl_poly_eval(ptr0, N, NUM2DBL(rb_ary_entry(x, i)));
@@ -141,6 +154,23 @@ static VALUE rb_gsl_poly_eval_singleton(VALUE klass, VALUE a, VALUE x)
       val = na_make_object(NA_DFLOAT, 1, shape, CLASS_OF(x));
       ptr1 = NA_PTR_TYPE(x, double*);
       ptr2 = NA_PTR_TYPE(val, double*);      
+#endif
+#ifdef GSL_1_11_LATER
+    } else if (rb_obj_is_kind_of(x, cgsl_complex)) {
+      Data_Get_Struct(x, gsl_complex, z);
+      zz = gsl_poly_complex_eval(ptr0, N, *z);
+      z = make_complex(GSL_REAL(zz), GSL_IMAG(zz));
+      if (flag == 1) gsl_vector_free(v);
+      return Data_Wrap_Struct(cgsl_complex, 0, free, z);
+    } else if (rb_obj_is_kind_of(x, cgsl_vector_complex)) {
+      Data_Get_Struct(x, gsl_vector_complex, vz);
+      vznew = gsl_vector_complex_alloc(vz->size);
+      for (i = 0; i < vz->size; i++) {
+	zz = gsl_poly_complex_eval(ptr0, N, gsl_vector_complex_get(vz, i));
+	gsl_vector_complex_set(vznew, i, zz);
+      }
+      if (flag == 1) gsl_vector_free(v);
+      return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vznew);
 #endif
     } else {
       rb_raise(rb_eTypeError, "Wrong argument type %s (A number, Array, GSL::Vector or NArray expected)",
@@ -153,7 +183,72 @@ static VALUE rb_gsl_poly_eval_singleton(VALUE klass, VALUE a, VALUE x)
   if (flag == 1) gsl_vector_free(v);
   return val;
 }
+#ifdef GSL_1_11_LATER
+static VALUE rb_gsl_complex_poly_complex_eval(VALUE a, VALUE b)
+{
+  gsl_vector_complex *coef, *zb, *vnew;
+  gsl_complex *zc;
+  gsl_complex z, *zx, *res;
+  VALUE ret;
+  size_t i, N;
+  int flag = 0;
+  if (rb_obj_is_kind_of(a, cgsl_vector_complex)) {
+    Data_Get_Struct(a, gsl_vector_complex, coef);
+    N = coef->size;
+    zc = (gsl_complex*) coef->data;
+  } else if (TYPE(a) == T_ARRAY) {
+    N = RARRAY_LEN(a);
+    zc = (gsl_complex*) malloc(sizeof(gsl_complex));
+    flag = 1;
+    for (i = 0; i < N; i++) {
+      Data_Get_Struct(rb_ary_entry(a, i), gsl_complex, zx);
+      zc[i] = *zx;
+    }
+  } else {
+    rb_raise(rb_eTypeError, "rb_gsl_complex_poly_complex_solve: wrong argument type %s (GSL::Vector::Complex or Array expected)\n", rb_class2name(CLASS_OF(a)));
+  }
 
+  switch (TYPE(b)) {
+  case T_FIXNUM:
+  case T_BIGNUM:
+  case T_FLOAT:
+    res = (gsl_complex*) malloc(sizeof(gsl_complex));
+    ret = Data_Wrap_Struct(cgsl_complex, 0, free, res);
+    GSL_SET_REAL(&z, NUM2DBL(b));
+    GSL_SET_IMAG(&z, 0.0);
+    *res = gsl_complex_poly_complex_eval(zc, coef->size, z);
+    break;
+  case T_ARRAY:
+    ret = rb_ary_new2(RARRAY_LEN(b));
+    for (i = 0; i < RARRAY_LEN(b); i++) {
+      Data_Get_Struct(rb_ary_entry(b, i), gsl_complex, zx);
+      res = (gsl_complex*) malloc(sizeof(gsl_complex));
+      *res = gsl_complex_poly_complex_eval(zc, N, *zx);
+      rb_ary_store(ret, i, Data_Wrap_Struct(cgsl_complex, 0, free, res));
+    }
+    break;
+  default:
+    if (rb_obj_is_kind_of(b, cgsl_complex)) {
+      res = (gsl_complex*) malloc(sizeof(gsl_complex));
+      ret = Data_Wrap_Struct(cgsl_complex, 0, free, res);
+      Data_Get_Struct(b, gsl_complex, zx);
+      *res = gsl_complex_poly_complex_eval(zc, N, *zx);
+    } else if (rb_obj_is_kind_of(b, cgsl_vector_complex)) {
+      Data_Get_Struct(b, gsl_vector_complex, zb);
+      vnew = gsl_vector_complex_alloc(zb->size);
+      ret = Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vnew);
+      for (i = 0; i < zb->size; i++) {
+	z = gsl_vector_complex_get(zb, i);
+	gsl_vector_complex_set(vnew, i, gsl_complex_poly_complex_eval(zc, N, z));
+      }
+    } else {
+      rb_raise(rb_eTypeError, "Wrong argument type %s.\n", rb_class2name(CLASS_OF(b)));
+    }
+  }
+  if (flag == 1) free(zc);
+  return ret;
+}
+#endif
 #endif
 
 static VALUE FUNCTION(rb_gsl_poly,eval)(VALUE obj, VALUE xx)
@@ -171,7 +266,12 @@ static VALUE FUNCTION(rb_gsl_poly,eval)(VALUE obj, VALUE xx)
   double *ptr1, *ptr2;
   size_t n;
 #endif
+#ifdef GSL_1_11_LATER
+  gsl_complex *z, zz;
+  gsl_vector_complex *vz, *vznew;
 #endif
+#endif
+
   Data_Get_Struct(obj, GSL_TYPE(gsl_poly), p);
   if (CLASS_OF(xx) == rb_cRange) xx = rb_gsl_range2ary(xx);
   switch (TYPE(xx)) {
@@ -181,8 +281,8 @@ static VALUE FUNCTION(rb_gsl_poly,eval)(VALUE obj, VALUE xx)
     return rb_float_new(FUNCTION(gsl_poly,eval)(p->data, p->size, NUM2DBL(xx)));
     break;
   case T_ARRAY:
-    ary = rb_ary_new2(RARRAY(xx)->len);
-    for (i = 0; i < RARRAY(xx)->len; i++) {
+    ary = rb_ary_new2(RARRAY_LEN(xx));
+    for (i = 0; i < RARRAY_LEN(xx); i++) {
       x = rb_ary_entry(xx, i);
       Need_Float(x);
       rb_ary_store(ary, i, rb_float_new(FUNCTION(gsl_poly,eval)(p->data, p->size, NUM2DBL(x))));
@@ -220,6 +320,23 @@ static VALUE FUNCTION(rb_gsl_poly,eval)(VALUE obj, VALUE xx)
 	}
       }
       return Data_Wrap_Struct(cgsl_matrix, 0, gsl_matrix_free, mnew);
+#ifdef BASE_DOUBLE
+#ifdef GSL_1_11_LATER
+    } else if (rb_obj_is_kind_of(xx, cgsl_complex)) {
+      Data_Get_Struct(xx, gsl_complex, z);
+      zz = gsl_poly_complex_eval(p->data, p->size, *z);
+      z = make_complex(GSL_REAL(zz), GSL_IMAG(zz));
+      return Data_Wrap_Struct(cgsl_complex, 0, free, z);
+    } else if (rb_obj_is_kind_of(xx, cgsl_vector_complex)) {
+      Data_Get_Struct(xx, gsl_vector_complex, vz);
+      vznew = gsl_vector_complex_alloc(vz->size);
+      for (i = 0; i < vz->size; i++) {
+	zz = gsl_poly_complex_eval(p->data, p->size, gsl_vector_complex_get(vz, i));
+	gsl_vector_complex_set(vznew, i, zz);
+      }
+      return Data_Wrap_Struct(cgsl_vector_complex, 0, gsl_vector_complex_free, vznew);
+#endif
+#endif
     } else {
       rb_raise(rb_eTypeError, "wrong argument type");
     }
@@ -267,8 +384,8 @@ static VALUE FUNCTION(rb_gsl_poly,eval2)(int argc, VALUE *argv, VALUE obj)
     return rb_float_new(FUNCTION(gsl_poly,eval)(p->data, size, NUM2DBL(xx)));
     break;
   case T_ARRAY:
-    ary = rb_ary_new2(RARRAY(xx)->len);
-    for (i = 0; i < RARRAY(xx)->len; i++) {
+    ary = rb_ary_new2(RARRAY_LEN(xx));
+    for (i = 0; i < RARRAY_LEN(xx); i++) {
       x = rb_ary_entry(xx, i);
       Need_Float(x);
       rb_ary_store(ary, i, rb_float_new(FUNCTION(gsl_poly,eval)(p->data, size, NUM2DBL(x))));
@@ -597,7 +714,7 @@ VALUE FUNCTION(rb_gsl_poly,complex_solve)(int argc, VALUE *argv, VALUE obj)
 
   switch (TYPE(argv[0])) {
   case T_ARRAY:
-    if (size < 0) size = RARRAY(argv[0])->len;
+    if (size < 0) size = RARRAY_LEN(argv[0]);
     a = gsl_vector_alloc(size);
     for (i = 0; i < size; i++) gsl_vector_set(a, i, NUMCONV2(rb_ary_entry(argv[0], i)));
     break;
@@ -917,7 +1034,7 @@ static VALUE rb_gsl_poly_dd_eval(VALUE obj, VALUE xxa, VALUE xx)
 					 NUM2DBL(xx)));
     break;
   case T_ARRAY:
-    size = RARRAY(xx)->len;
+    size = RARRAY_LEN(xx);
     ary = rb_ary_new2(size);
     for (i = 0; i < size; i++) {
       x = rb_ary_entry(xx, i);
@@ -1143,7 +1260,7 @@ GSL_TYPE(gsl_poly)* FUNCTION(get_poly,get)(VALUE obj, int *flag)
   size_t i;
   switch (TYPE(obj)) {
   case T_ARRAY:
-    p = FUNCTION(gsl_vector,alloc)(RARRAY(obj)->len);
+    p = FUNCTION(gsl_vector,alloc)(RARRAY_LEN(obj));
     for (i = 0; i < p->size; i++) FUNCTION(gsl_vector,set)(p, i, (BASE) NUM2DBL(rb_ary_entry(obj, i)));
     *flag = 1;
     break;
@@ -1408,7 +1525,11 @@ static VALUE FUNCTION(rb_gsl_poly,info)(VALUE obj)
   char buf[256];
   Data_Get_Struct(obj, GSL_TYPE(gsl_poly), v);
   sprintf(buf, "Class:      %s\n", rb_class2name(CLASS_OF(obj)));
+#ifdef RUBY_1_9_LATER
+  sprintf(buf, "%sSuperClass: %s\n", buf, rb_class2name(RCLASS_SUPER(CLASS_OF(obj))));
+#else
   sprintf(buf, "%sSuperClass: %s\n", buf, rb_class2name(RCLASS(CLASS_OF(obj))->super));
+#endif
   sprintf(buf, "%sOrder:      %d\n", buf, (int) v->size-1);
   return rb_str_new2(buf);
 }
@@ -1503,6 +1624,86 @@ static VALUE rb_gsl_poly_wfit(int argc, VALUE *argv, VALUE obj)
 }
 #endif
 
+#ifdef BASE_DOUBLE
+#ifdef GSL_1_13_LATER
+static VALUE rb_gsl_poly_eval_derivs_singleton(int argc, VALUE *argv, VALUE klass)
+{
+  VALUE ary;
+  gsl_vector *v = NULL, *v2 = NULL;
+  size_t i, lenc, lenres;
+#ifdef HAVE_NARRAY_H
+  struct NARRAY *na;
+  double *ptr1, *ptr2;
+  int shape[1];
+#endif
+
+  if (argc < 2) rb_raise(rb_eArgError, "Wrong number of arguments (%d for >= 2)", argc);
+  if (rb_obj_is_kind_of(argv[0], rb_cArray)) {
+    v = gsl_vector_alloc(RARRAY_LEN(argv[0]));
+    lenc = v->size;
+    for (i = 0; i < lenc; i++) {
+      gsl_vector_set(v, i, NUM2DBL(rb_ary_entry(argv[0], i)));
+    }
+    if (argc == 2) lenres = lenc + 1;
+    else lenres = FIX2INT(argv[2]);
+    v2 = gsl_vector_alloc(lenres);
+    gsl_poly_eval_derivs(v->data, lenc, NUM2DBL(argv[1]), v2->data, lenres);
+    ary = rb_ary_new2(lenres);
+    for (i = 0; i < lenres; i++) {
+      rb_ary_store(ary, i, rb_float_new(gsl_vector_get(v2, i)));
+    }
+    gsl_vector_free(v2);
+    gsl_vector_free(v);
+    return ary;
+  }
+  if (rb_obj_is_kind_of(argv[0], cgsl_vector)) {
+    Data_Get_Struct(argv[0], gsl_vector, v);
+    lenc = v->size;
+    if (argc == 2) lenres = lenc + 1;
+    else lenres = FIX2INT(argv[2]);
+    v2 = gsl_vector_alloc(lenres);
+    gsl_poly_eval_derivs(v->data, lenc, NUM2DBL(argv[1]), v2->data, lenres);
+    return Data_Wrap_Struct(cgsl_poly, 0, gsl_vector_free, v2);
+  }
+#ifdef HAVE_NARRAY_H
+  if (NA_IsNArray(argv[0])) {
+    GetNArray(argv[0], na);
+    ptr1 = (double*) na->ptr;
+    lenc = na->total;
+    if (argc == 2) lenres = lenc + 1;
+    else lenres = FIX2INT(argv[2]);
+    shape[0] = lenres;
+    ary = na_make_object(NA_DFLOAT, na->rank, shape, CLASS_OF(argv[0]));
+    ptr2 = NA_PTR_TYPE(ary,double*);
+    gsl_poly_eval_derivs(ptr1, lenc, NUM2DBL(argv[1]), ptr2, lenres);
+    return ary;
+  }
+#endif
+  return Qnil;  // Never comes here
+}
+static VALUE rb_gsl_poly_eval_derivs(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_vector *v, *v2;
+  size_t lenc, lenres;
+  Data_Get_Struct(obj, gsl_vector, v);
+  lenc = v->size;
+  switch (argc) {
+  case 1:
+    lenres = lenc + 1;
+    break;
+  case 2:
+    lenres = FIX2INT(argv[1]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "Wrong number of arguments (%d for > 1)", argc);
+  }
+  v2 = gsl_vector_alloc(lenres);
+  gsl_poly_eval_derivs(v->data, lenc, NUM2DBL(argv[0]), v2->data, lenres);
+  return Data_Wrap_Struct(cgsl_poly, 0, gsl_vector_free, v2);
+}
+#endif
+#endif
+
 void FUNCTION(Init_gsl_poly,init)(VALUE module)
 {
 #ifdef BASE_DOUBLE
@@ -1592,8 +1793,12 @@ void FUNCTION(Init_gsl_poly,init)(VALUE module)
 #endif
 
 #ifdef BASE_DOUBLE
-  rb_define_singleton_method(cgsl_poly, "eval", rb_gsl_poly_eval_singleton, 2);
+  //  rb_define_singleton_method(cgsl_poly, "eval", rb_gsl_poly_eval_singleton, 2);
   rb_define_method(cgsl_poly, "to_i", rb_gsl_poly_to_i, 0);
+#ifdef GSL_1_11_LATER
+  rb_define_singleton_method(cgsl_poly, "complex_eval", rb_gsl_poly_eval_singleton, 2);
+  rb_define_method(cgsl_vector_complex, "eval", rb_gsl_complex_poly_complex_eval, 1);
+#endif
 #ifdef GSL_1_1_LATER
   cgsl_poly_dd = rb_define_class_under(cgsl_poly, "DividedDifference", cgsl_poly);
   cgsl_poly_taylor = rb_define_class_under(cgsl_poly, "Taylor", cgsl_poly);
@@ -1632,9 +1837,6 @@ void FUNCTION(Init_gsl_poly,init)(VALUE module)
   rb_define_method(GSL_TYPE(cgsl_poly), "-@", FUNCTION(rb_gsl_poly,uminus), 0);
   rb_define_method(GSL_TYPE(cgsl_poly), "+@", FUNCTION(rb_gsl_poly,uplus), 0);
 
-  rb_define_alias(GSL_TYPE(cgsl_poly), "up", "unshift");
-  rb_define_alias(GSL_TYPE(cgsl_poly), "down", "shift");
-
   rb_define_method(GSL_TYPE(cgsl_poly), "coerce", 
 		   FUNCTION(rb_gsl_poly,coerce), 1);
   rb_define_method(GSL_TYPE(cgsl_poly), "to_gv", FUNCTION(rb_gsl_poly,to_gv), 0);
@@ -1653,6 +1855,12 @@ void FUNCTION(Init_gsl_poly,init)(VALUE module)
 		   FUNCTION(rb_gsl_poly,fit), -1);
   rb_define_singleton_method(GSL_TYPE(cgsl_poly), "wfit", 
 		   FUNCTION(rb_gsl_poly,wfit), -1);
+
+#ifdef GSL_1_13_LATER
+  rb_define_singleton_method(cgsl_poly, "eval_derivs", rb_gsl_poly_eval_derivs_singleton, -1);
+  rb_define_method(cgsl_vector, "eval_derivs", rb_gsl_poly_eval_derivs, -1);
+#endif
+
 #endif
 }
 
