@@ -1,7 +1,7 @@
 /*
   siman.c
   Ruby/GSL: Ruby extension library for GSL (GNU Scientific Library)
-    (C) Copyright 2001-2004 by Yoshiki Tsunesada
+    (C) Copyright 2001-2006 by Yoshiki Tsunesada
 
   Ruby/GSL is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License.
@@ -118,7 +118,7 @@ static double rb_gsl_siman_Efunc_t(void *data)
   ss = (siman_solver *) data;
   proc = (VALUE) ss->proc_efunc;
   params = Data_Wrap_Struct(cgsl_vector, 0, NULL, ss->vx);
-  result = rb_funcall(proc, ID_call, 1, params);
+  result = rb_funcall(proc, RBGSL_ID_call, 1, params);
   return NUM2DBL(result);
 }
 
@@ -224,7 +224,7 @@ static void rb_gsl_siman_print_t(void *data)
   proc = ss->proc_print;
   if (NIL_P(proc)) return;
   params = Data_Wrap_Struct(cgsl_vector, 0, NULL, ss->vx);
-  rb_funcall(proc, ID_call, 1, params);
+  rb_funcall(proc, RBGSL_ID_call, 1, params);
 }
 
 static VALUE rb_gsl_siman_print_set(int argc, VALUE *argv, VALUE obj)
@@ -293,7 +293,7 @@ static void rb_gsl_siman_step_t(const gsl_rng *r, void *data, double step_size)
   proc = (VALUE) ss->proc_step;
   rng = Data_Wrap_Struct(cgsl_rng, 0, NULL, (gsl_rng *) r);
   params = Data_Wrap_Struct(cgsl_vector, 0, NULL, ss->vx);
-  rb_funcall(proc, ID_call, 3, rng, params, rb_float_new(step_size));
+  rb_funcall(proc, RBGSL_ID_call, 3, rng, params, rb_float_new(step_size));
 }
 
 static VALUE rb_gsl_siman_step_set(int argc, VALUE *argv, VALUE obj)
@@ -363,7 +363,7 @@ static double rb_gsl_siman_metric_t(void *data, void *yp)
   proc = ss->proc_metric;
   vxp = Data_Wrap_Struct(cgsl_vector, 0, NULL, ss->vx);
   vyp = Data_Wrap_Struct(cgsl_vector, 0, NULL, ssy->vx);
-  result = rb_funcall(proc, ID_call, 2, vxp, vyp);
+  result = rb_funcall(proc, RBGSL_ID_call, 2, vxp, vyp);
   return NUM2DBL(result);
 }
 
