@@ -84,7 +84,7 @@ static VALUE rb_gsl_vector_linspace(int argc, VALUE *argv, VALUE klass)
     /* do nothing */
     break;
   default:
-    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)");
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
     break;
   }
   Need_Float(argv[0]);   Need_Float(argv[1]); 
@@ -104,7 +104,7 @@ static VALUE rb_gsl_vector_logspace(int argc, VALUE *argv, VALUE klass)
     /* do nothing */
     break;
   default:
-    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)");
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
     break;
   }
   Need_Float(argv[0]);   Need_Float(argv[1]); 
@@ -124,7 +124,7 @@ static VALUE rb_gsl_vector_logspace2(int argc, VALUE *argv, VALUE klass)
     /* do nothing */
     break;
   default:
-    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)");
+    rb_raise(rb_eArgError, "wrong number of arguments (%d for 2 or 3)", argc);
     break;
   }
   Need_Float(argv[0]);   Need_Float(argv[1]); 
@@ -298,7 +298,7 @@ VALUE rb_ary_to_gv0(VALUE ary)
 {
   gsl_vector *v = NULL;
   size_t i, size;
-  size = RARRAY(ary)->len;
+  size = RARRAY_LEN(ary);
   v = gsl_vector_alloc(size);
   if (v == NULL) rb_raise(rb_eNoMemError, "gsl_vector_alloc failed");
   for (i = 0; i < size; i++) {
@@ -311,7 +311,7 @@ VALUE rb_ary_to_gv(VALUE klass, VALUE ary)
 {
   gsl_vector *v = NULL;
   size_t i, size;
-  size = RARRAY(ary)->len;
+  size = RARRAY_LEN(ary);
   v = gsl_vector_alloc(size);
   if (v == NULL) rb_raise(rb_eNoMemError, "gsl_vector_alloc failed");
   for (i = 0; i < size; i++) {
@@ -330,7 +330,7 @@ VALUE rb_gsl_range_to_gv(VALUE obj)
   if (RTEST(rb_ivar_get(obj, rb_gsl_id_excl))) n = en - beg;
   else n = en - beg + 1;
   v = gsl_vector_alloc(n);
-  for (i = 0; i < n; i++) gsl_vector_set(v, i, beg + i);
+  for (i = 0; i < n; i++) gsl_vector_set(v, i, beg + (int)i);
   return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, v);
 }
 gsl_vector_view* gsl_vector_view_alloc()
@@ -707,7 +707,7 @@ static void draw_vector_array(VALUE ary, FILE *fp)
   VALUE vx;
   size_t j, n, stridex, stridey, stridez;
   int flag = 0;
-  switch (RARRAY(ary)->len) {
+  switch (RARRAY_LEN(ary)) {
   case 1:
     flag = 1;
     ptry = get_vector_ptr(rb_ary_entry(ary, 0), &stridey, &n);
@@ -732,7 +732,7 @@ static void draw_vector_array(VALUE ary, FILE *fp)
     break;
   default:
     rb_raise(rb_eRuntimeError, "wrong array length (%d for 1 or 2)", 
-	     RARRAY(ary)->len);
+	     (int) RARRAY_LEN(ary));
     break;
   }
   switch (flag) {
@@ -839,20 +839,11 @@ static VALUE rb_gsl_vector_plot2(int argc, VALUE *argv, VALUE obj)
   return Qtrue;
 }
 
-static VALUE rb_gsl_vector_sqrt(VALUE obj)
-{
-  gsl_vector *v = NULL, *vnew = NULL;
-  size_t i;
-  Data_Get_Vector(obj, v);
-  vnew = gsl_vector_alloc(v->size);
-  for (i = 0; i < v->size; i++) gsl_vector_set(vnew, i, sqrt(gsl_vector_get(v, i)));
-  return Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, vnew);  
-}
-
 static VALUE rb_gsl_vector_normalize(int argc, VALUE *argv, VALUE obj)
 {
   gsl_vector *v = NULL, *vnew = NULL;
-  double dnrm2, nrm;
+  //  double mean;
+  double nrm;
   switch (argc) {
   case 0:
     nrm = 1.0;
@@ -867,15 +858,20 @@ static VALUE rb_gsl_vector_normalize(int argc, VALUE *argv, VALUE obj)
   }
   Data_Get_Vector(obj, v);
   vnew = make_vector_clone(v);
-  dnrm2 = gsl_blas_dnrm2(vnew);
-  gsl_vector_scale(vnew, sqrt(nrm)/dnrm2);
+  /*  mean = gsl_stats_mean(v->data, v->stride, v->size);
+  gsl_vector_add_constant(vnew, -mean);
+  sd = gsl_stats_sd(vnew->data, vnew->stride, vnew->size);  
+  gsl_vector_scale(vnew, sqrt(nrm)/sd);*/
+  gsl_vector_scale(vnew, nrm/gsl_blas_dnrm2(v));
   return Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, vnew);  
 }
 
 static VALUE rb_gsl_vector_normalize_bang(int argc, VALUE *argv, VALUE obj)
 {
   gsl_vector *v = NULL;
-  double dnrm2, nrm;
+  double mean;  
+  double nrm;
+  double factor;
   switch (argc) {
   case 0:
     nrm = 1.0;
@@ -889,8 +885,12 @@ static VALUE rb_gsl_vector_normalize_bang(int argc, VALUE *argv, VALUE obj)
     break;
   }
   Data_Get_Vector(obj, v);
-  dnrm2 = gsl_blas_dnrm2(v);
-  gsl_vector_scale(v, sqrt(nrm)/dnrm2);
+  mean = gsl_stats_mean(v->data, v->stride, v->size);
+  /*  gsl_vector_add_constant(v, -mean);  
+  sd = gsl_stats_sd(v->data, v->stride, v->size);  
+  gsl_vector_scale(v, sqrt(nrm)/sd);*/
+  factor = nrm/gsl_blas_dnrm2(v);
+  gsl_vector_scale(v, factor);
   return obj;
 }
 
@@ -991,13 +991,27 @@ static VALUE rb_gsl_vector_ceil(VALUE obj)
 }
 
 #ifdef HAVE_ROUND
-double round(double);
-static VALUE rb_gsl_vector_round(VALUE obj)
+double round(double x);
+#define rb_gsl_round_native round
+#else
+static double rb_gsl_round_native(double x)
 {
-  return rb_gsl_vector_xxx(obj, round);
+  if(!gsl_finite(x)) {
+    return x; // nan, +inf, -inf
+  } else if(x > 0.0) {
+    return floor(x+0.5);
+  } else if(x < 0.0) {
+    return ceil(x-0.5);
+  }
+  return x; // +0 or -0
 }
 #endif
 
+static VALUE rb_gsl_vector_round(VALUE obj)
+{
+  return rb_gsl_vector_xxx(obj, rb_gsl_round_native);
+}
+
 static VALUE rb_gsl_vector_dB(VALUE obj)
 {
   gsl_vector *v, *vnew;
@@ -1239,6 +1253,98 @@ static VALUE rb_gsl_vector_clip(int argc, VALUE *argv, VALUE obj)
   return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);
 }
 
+static VALUE rb_gsl_vector_amp_phase(VALUE obj)
+{
+  gsl_vector *v;
+  gsl_vector *amp, *phase;
+  double re, im;
+  VALUE vamp, vphase;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector, v);
+  amp = gsl_vector_alloc(v->size/2);
+  phase = gsl_vector_alloc(v->size/2);
+  gsl_vector_set(amp, 0, gsl_vector_get(v, 0));
+  gsl_vector_set(phase, 0, 0);  
+  gsl_vector_set(amp, amp->size-1, gsl_vector_get(v, v->size-1));
+  gsl_vector_set(phase, phase->size-1, 0);    
+  for (i = 1; i < v->size-1; i+=2) {
+    re = gsl_vector_get(v, i);
+    im = gsl_vector_get(v, i+1);    
+    gsl_vector_set(amp, i/2+1, sqrt(re*re + im*im));
+    gsl_vector_set(phase, i/2+1, atan2(im, re));
+  }
+  vamp = Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, amp);
+  vphase = Data_Wrap_Struct(VECTOR_ROW_COL(obj), 0, gsl_vector_free, phase);
+  return rb_ary_new3(2, vamp, vphase);
+}
+
+
+static VALUE rb_gsl_vector_clean(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_vector *v = NULL, *vnew = NULL;
+  double eps = 1e-10;
+  size_t n, i;
+  switch (argc) {
+  case 0:
+    /* do nothing */
+    break;
+  case 1:
+    Need_Float(argv[0]);
+    eps = NUM2DBL(argv[0]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "too many arguments (%d for 0 or 1)", argc);
+    break;
+  }
+  Data_Get_Struct(obj, gsl_vector, v);
+  vnew = make_vector_clone(v);
+  n = v->size;
+  for (i = 0; i < n; i++) if (fabs(vnew->data[i]) < eps) vnew->data[i] = 0.0;
+  return Data_Wrap_Struct(cgsl_vector, 0, gsl_vector_free, vnew);;
+}
+
+static VALUE rb_gsl_vector_clean_bang(int argc, VALUE *argv, VALUE obj)
+{
+  gsl_vector *v = NULL;
+  double eps = 1e-10;
+  size_t n, i;
+  switch (argc) {
+  case 0:
+    /* do nothing */
+    break;
+  case 1:
+    Need_Float(argv[0]);
+    eps = NUM2DBL(argv[0]);
+    break;
+  default:
+    rb_raise(rb_eArgError, "too many arguments (%d for 0 or 1)", argc);
+    break;
+  }
+  Data_Get_Struct(obj, gsl_vector, v);
+  n = v->size;
+  for (i = 0; i < n; i++) if (fabs(v->data[i]) < eps) v->data[i] = 0.0;
+  return obj;
+}
+
+VALUE rb_gsl_pow(VALUE obj, VALUE xx, VALUE nn);
+VALUE rb_gsl_vector_pow(VALUE obj, VALUE p)
+{
+  return rb_gsl_pow(Qnil, obj, p);
+}
+
+VALUE rb_gsl_vector_pow_bang(VALUE obj, VALUE pp)
+{
+  gsl_vector *v;
+  double p;
+  size_t i;
+  Data_Get_Struct(obj, gsl_vector, v);
+  p = NUM2DBL(pp);
+  for (i = 0; i < v->size; i++) {
+    gsl_vector_set(v, i, pow(gsl_vector_get(v, i), p));
+  }
+  return obj;
+}
+
 void Init_gsl_vector(VALUE module)
 {
   rb_define_singleton_method(cgsl_vector, "linspace", rb_gsl_vector_linspace, -1);
@@ -1283,8 +1389,6 @@ void Init_gsl_vector(VALUE module)
 
   /*****/
 
-  rb_define_method(cgsl_vector, "sqrt", rb_gsl_vector_sqrt, 0);
-
   rb_define_method(cgsl_vector, "normalize", rb_gsl_vector_normalize, -1);
   rb_define_method(cgsl_vector, "normalize!", rb_gsl_vector_normalize_bang, -1);
 
@@ -1297,9 +1401,7 @@ void Init_gsl_vector(VALUE module)
 
   rb_define_method(cgsl_vector, "floor", rb_gsl_vector_floor, 0);
   rb_define_method(cgsl_vector, "ceil", rb_gsl_vector_ceil, 0);
-#ifdef HAVE_ROUND
   rb_define_method(cgsl_vector, "round", rb_gsl_vector_round, 0);
-#endif
 
   rb_define_method(cgsl_vector, "dB", rb_gsl_vector_dB, 0);
 
@@ -1318,5 +1420,14 @@ void Init_gsl_vector(VALUE module)
   rb_define_method(cgsl_vector, "center", rb_gsl_vector_center, 0);
   rb_define_method(cgsl_vector, "clip", rb_gsl_vector_clip, -1);
 
+  rb_define_method(cgsl_vector, "amp_phase", rb_gsl_vector_amp_phase, 0);
+  rb_define_method(cgsl_vector, "clean", rb_gsl_vector_clean, -1);
+  rb_define_method(cgsl_vector, "clean!", rb_gsl_vector_clean_bang, -1);
+  
+  rb_define_method(cgsl_vector, "pow", rb_gsl_vector_pow, 1);
+  rb_define_alias(cgsl_vector, "**", "pow");
+  
+  rb_define_method(cgsl_vector, "pow!", rb_gsl_vector_pow_bang, 1);  
+  
   Init_gsl_vector_init(module);
 }
