@@ -23,7 +23,7 @@ m = GSL::MODE_DEFAULT
   TEST_SF(s, "GSL::Sf::bessel_Jn_e", "(2, 15000.0)", -0.0020455820181216382666, TEST_TOL4, GSL::SUCCESS)
   TEST_SF(s, "GSL::Sf::bessel_Jn_e", "(0, 1.0e+10)", 2.1755917502468917269e-06, TEST_SQRT_TOL0, GSL::SUCCESS)
   TEST_SF(s, "GSL::Sf::bessel_Jn_e", "(1, 1.0e+10)", -7.676508175684157103e-06, TEST_TOL4, GSL::SUCCESS)
-  TEST_SF(s, "GSL::Sf::bessel_Jn_e", "(0, 20000)", -7.676508175684157103e-06, TEST_TOL4, GSL::SUCCESS)
+#  TEST_SF(s, "GSL::Sf::bessel_Jn_e", "(0, 20000)", -7.676508175684157103e-06, TEST_TOL4, GSL::SUCCESS)
   TEST_SF(s, "GSL::Sf::bessel_Y0_e", "(0.1)",         -1.5342386513503668441,    TEST_TOL0, GSL::SUCCESS)
   TEST_SF(s, "GSL::Sf::bessel_Y0_e", "(2)",            0.5103756726497451196,    TEST_TOL0, GSL::SUCCESS)
   TEST_SF(s, "GSL::Sf::bessel_Y0_e", "(256.0)",       -0.03381290171792454909 ,  TEST_TOL0, GSL::SUCCESS)
